/*
 * Judge.java
 *
 * Created on 20 janvier 2004, 15:39
 */

package simulations.core;

import simulations.core.history.*;
import simulations.gui.coalitions.CoalitionView;
import simulations.core.strategy.completeStrategy.CompleteStrategy;
import simulations.core.strategy.PlayerIntElement;
import simulations.core.exceptions.*;
import simulations.gui.coalitions.CoalitionData;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Map.Entry;
import java.util.Iterator;
import java.util.Calendar;
import java.util.GregorianCalendar;

import java.io.File;
import java.io.IOException;

/** This class describes the judge of the game (the mediator)
 * @author tom
 */
public class Judge extends java.lang.Thread {
    
    /** The game **/
    private Game game;
    
    /** The current turn **/
    private int currentTurn;
    
    /** The max number of turn **/
    private final int maxTurn = 10000;
    
    /** The writer of the history **/
    private HistoryWriter historyWriter;
    
    /** The list of scores update to do at the end of the turn **/
    private ArrayList scoresUpdateToDo;
    
    /** The game history **/
    private GameHistory gameHistory;
    
    /** List of the three last turn **/
    private ArrayList threeLastTurn;
    
    /** List of the playerList **/
    private ArrayList playerListList;
    
    /** The dialog box of coalition formation **/
    private CoalitionView coalitionView;
    
    /** The coalitions represents by disconnected sets of players **/
    private ArrayList coalitionList;
    
    /** The statistics of the game **/
    private ArrayList statisticsList;
    
    /** The turn description **/
    private String turnDescription;
    
    /** The classement of the game **/
    private ArrayList classement;
    
    /** The list of all possibles coalitions **/
    private ArrayList allCoalitionsList;
    
    /** The list of activated coalitions since the beginning **/
    private ArrayList coalitionListList;
    
    /** The coalitions ranking **/
    private Hashtable coalitionsRanking;
    
    /** Creates a new instance of Judge
     * @param game The game that the judge will manage.
     */
    public Judge(Game game) {
        this.game=game;
        scoresUpdateToDo=new ArrayList();
        playerListList = new ArrayList();
        coalitionList = new ArrayList();
        statisticsList = new ArrayList();
        classement = new ArrayList();
        allCoalitionsList = new ArrayList();
        coalitionListList = new ArrayList();
        coalitionsRanking = new Hashtable();
        turnDescription="";
        currentTurn=0;
    }
    
    /** Returns true if p1 and p2 are in the samecoalition
     * @param p1 Player 1
     * @param p2 Player 2
     * @param l The players turn list
     **/
    private boolean sameCoalition(Player p1, Player p2, ArrayList l) {
        // get player turn of each player
        //System.out.println("sameCoalition : ");
        //System.out.println(p1.toString()+" "+p2.toString()+" "+l.toString());
        PlayerTurn pt1 = null;
        PlayerTurn pt2 = null;
        Iterator it = l.iterator();
        //System.out.println("DANS SAMECOALITION2");
        while (it.hasNext()) {
            PlayerTurn pt = (PlayerTurn)it.next();
            //System.out.println("PlayerTurn de "+pt.getPlayer());
            if (pt.getPlayer().equals(p1)) {
                pt1 = pt;
            } else if (pt.getPlayer().equals(p2)) {
                pt2 = pt;
            }
        }
        //System.out.println("DANS SAMECOALITION2______________1");
        // switch coalition definition
        switch (game.getCoalitionDefinition()) {
            case 0 :
                // if one attack the other
                //System.out.println("DANS SAMECOALITION2______________CASE0");
                if (pt1.getAttackedPlayer().equals(p2)) {
                    return false;
                }
                if (pt2.getAttackedPlayer().equals(p1)) {
                    return false;
                }
                return true;
            case 1 :
                // if one attack the other
                //System.out.println("DANS SAMECOALITION2______________CASE1");
                if (pt1.getAttackedPlayer().equals(p2)) {
                    return false;
                }
                if (pt2.getAttackedPlayer().equals(p1)) {
                    return false;
                }
                // if one support an attack on the other
                it = l.iterator();
                while (it.hasNext()) {
                    PlayerTurn pt = (PlayerTurn)it.next();
                    // if the attacked is one of the two we compare we look if one
                    // of them has supported an attack on the other
                    if (pt.getAttackedPlayer().equals(p1)) {
                        Iterator it2 = pt.getPlayersAcceptingSupport().iterator();
                        while (it2.hasNext()) {
                            Player p = (Player)it2.next();
                            if (p.equals(p2)) {
                                return false;
                            }
                        }
                    } else if (pt.getAttackedPlayer().equals(p2)) {
                        Iterator it2 = pt.getPlayersAcceptingSupport().iterator();
                        while (it2.hasNext()) {
                            Player p = (Player)it2.next();
                            if (p.equals(p1)) {
                                return false;
                            }
                        }
                    }
                }
                return true;
            case 2 :
                //System.out.println("DANS SAMECOALITION2______________CASE2");
                if (pt1.getAttackedPlayer().equals(p2)) {
                    return false;
                }
                if (pt2.getAttackedPlayer().equals(p1)) {
                    return false;
                }
                // if they help each others
                if (!(pt1.getPlayersAcceptingSupport().contains(p2))) {
                    return false;
                }
                if (!(pt2.getPlayersAcceptingSupport().contains(p1))) {
                    return false;
                }
                // if one support an attack on the other
                it = l.iterator();
                while (it.hasNext()) {
                    PlayerTurn pt = (PlayerTurn)it.next();
                    // if the attacked is one of the two we compare we look if one
                    // of them has supported an attack on the other
                    if (pt.getAttackedPlayer().equals(p1)) {
                        Iterator it2 = pt.getPlayersAcceptingSupport().iterator();
                        while (it2.hasNext()) {
                            Player p = (Player)it2.next();
                            if (p.equals(p2)) {
                                return false;
                            }
                        }
                    } else if (pt.getAttackedPlayer().equals(p2)) {
                        Iterator it2 = pt.getPlayersAcceptingSupport().iterator();
                        while (it2.hasNext()) {
                            Player p = (Player)it2.next();
                            if (p.equals(p1)) {
                                return false;
                            }
                        }
                    }
                }
                return true;
            case 3 :
                // 1 + attack same player
                if (pt1.getAttackedPlayer().equals(p2)) {
                    return false;
                }
                if (pt2.getAttackedPlayer().equals(p1)) {
                    return false;
                }
                // if one support an attack on the other
                it = l.iterator();
                while (it.hasNext()) {
                    PlayerTurn pt = (PlayerTurn)it.next();
                    // if the attacked is one of the two we compare we look if one
                    // of them has supported an attack on the other
                    if (pt.getAttackedPlayer().equals(p1)) {
                        Iterator it2 = pt.getPlayersAcceptingSupport().iterator();
                        while (it2.hasNext()) {
                            Player p = (Player)it2.next();
                            if (p.equals(p2)) {
                                return false;
                            }
                        }
                    } else if (pt.getAttackedPlayer().equals(p2)) {
                        Iterator it2 = pt.getPlayersAcceptingSupport().iterator();
                        while (it2.hasNext()) {
                            Player p = (Player)it2.next();
                            if (p.equals(p1)) {
                                return false;
                            }
                        }
                    }
                }
                if (!(pt1.getAttackedPlayer().equals(pt2.getAttackedPlayer()))) {
                    return false;
                }
                return true;
            case 4 : 
                // 2 + Attack same player
                //System.out.println("DANS SAMECOALITION2______________CASE2");
                if (pt1.getAttackedPlayer().equals(p2)) {
                    return false;
                }
                if (pt2.getAttackedPlayer().equals(p1)) {
                    return false;
                }
                if (!(pt1.getAttackedPlayer().equals(pt2.getAttackedPlayer()))) {
                    return false;
                }
                // if they help each others
                if (!(pt1.getPlayersAcceptingSupport().contains(p2))) {
                    return false;
                }
                if (!(pt2.getPlayersAcceptingSupport().contains(p1))) {
                    return false;
                }
                // if one support an attack on the other
                it = l.iterator();
                while (it.hasNext()) {
                    PlayerTurn pt = (PlayerTurn)it.next();
                    // if the attacked is one of the two we compare we look if one
                    // of them has supported an attack on the other
                    if (pt.getAttackedPlayer().equals(p1)) {
                        Iterator it2 = pt.getPlayersAcceptingSupport().iterator();
                        while (it2.hasNext()) {
                            Player p = (Player)it2.next();
                            if (p.equals(p2)) {
                                return false;
                            }
                        }
                    } else if (pt.getAttackedPlayer().equals(p2)) {
                        Iterator it2 = pt.getPlayersAcceptingSupport().iterator();
                        while (it2.hasNext()) {
                            Player p = (Player)it2.next();
                            if (p.equals(p1)) {
                                return false;
                            }
                        }
                    }
                }
                return true;
        }
        return false;
    }
    
    /** Returns true if p is in the coalition coal
     * @param coal The coalition
     * @param p The player
     * @param l The players turns list
     **/
    private boolean sameCoalition(ArrayList coal, Player p, ArrayList l) {
        if (simulations.SimulationParameters.debug) {
            System.out.println("sameCoalition : ");
            System.out.println(coal.toString()+" "+p.toString()+" "+l.toString());
        }
        Iterator it = coal.iterator();
        while (it.hasNext()) {
            Player p2 = (Player)it.next();
            //System.out.println("Appel a SAMECOALITION2");
            if (!(sameCoalition(p2, p, l))) {
                if (simulations.SimulationParameters.debug) {
                    System.out.println("non");
                }
                return false;
            }
        }
        if (simulations.SimulationParameters.debug) {
            System.out.println("oui");
        }
        return true;
    }
    
    /** Returns true if the players list coal is a coalition
     * @param coal the players list to test
     * @param l The players turns list
     **/
    private boolean isCoalition(ArrayList coal, ArrayList l) {
        if (simulations.SimulationParameters.debug) {
            System.out.println(coal.toString()+" coal ?");
        }
        int i=0;
        for(int j=0;j<coal.size();j++) {
            Player p = (Player)coal.remove(i);
            if (!(sameCoalition(coal,p,l))) {
                coal.add(i,p);
                if (simulations.SimulationParameters.debug) {
                    System.out.println("Non");
                }
                return false;
            }
            coal.add(i,p);
            i++;
        }
        if (simulations.SimulationParameters.debug) {
            System.out.println("Oui");
        }
        return true;
    }
    
    /** Returns the list of all possibles coalitions
     * @returns the list of all possibles coalitions
     **/
    private ArrayList getAllCoalitions() {
        ArrayList list = getAllCoalitionsOf2Players();
        for(int size=0;size<(game.getPlayerList().size()-2);size++) {
            incrementCoalitionList(list);
        }
        // remove coalition of one player!
        for(int i=0;i<list.size();i++) {
            ArrayList l = (ArrayList)list.get(i);
            if (l.size()==1) {
                list.remove(i);
                i--;
            } else {
                break;
            }
        }
        // grand coalition can't be formed
        return list;
    }
    
    /** This method increase the size of the coalition list by adding a player
     * in each entries of maximum size
     * @param list the coalition list to increas
     **/
    private void incrementCoalitionList(ArrayList list) {
        int eltSize = ((ArrayList)list.get(list.size()-1)).size();
        for(int i=0;i<list.size();i++) {
            ArrayList l = (ArrayList)list.get(i);
            if (l.size()<eltSize) {
                continue;
            }
            Iterator it2 = game.getPlayerList().iterator();
            while (it2.hasNext()) {
                ArrayList l2 = new ArrayList();
                // rebuild list                
                Iterator it3 = l.iterator();
                while (it3.hasNext()) {
                    Player p = (Player)it3.next();
                    l2.add(p);
                }
                // add the new player
                Player pl = (Player)it2.next();
                if (!(l2.contains(pl))) {
                    l2.add(pl);
                    if (!(coalitionContains(list,l2))) {
                        list.add(l2);
                        System.out.println("adding "+l2.toString());
                    }
                }
            }
        }
    }
    
    /** Returns true if the coalition list list contains the coalition l
     * @param list The coalition list
     * @param l The coalition
     * @return true if the coalition list list contains the coalition l
     **/
    private boolean coalitionContains(ArrayList list, ArrayList l) {
        for(int i=0;i<list.size();i++) {
            ArrayList l2 = (ArrayList)list.get(i);
            if (l2.size()!=l.size()) {
                continue;
            }
            if (l2.containsAll(l)) {
                return true;
            }
        }
        return false;
    }
    
    /** Returns the list of all possibles coalitions of size size
     * @returns the list of all possibles coalitions of size size
     **/
    private ArrayList getAllCoalitionsOf2Players() {
        ArrayList list = new ArrayList();
        Iterator it = game.getPlayerList().iterator();
        while (it.hasNext()) {
            ArrayList l = new ArrayList();
            Player p = (Player)it.next();
            l.add((Player)p.clone());
            list.add(l);
        }
        return list;
    }
    
    /** Returns true if the list l contains a dead player
     * @param l The list
     * @return true if the list l contains a dead player
     **/
    private boolean containDeadPlayer(ArrayList l) {
        if (game.getRemainingPlayer().containsAll(l)) {
            return false;
        }
        return true;
    }
    
    private boolean isCandidate(Player p1, Player p2, ArrayList l) {
        return true;
    }
    
    /** This method update the sets of coalitions considering
     * the last turn turn
     * @param turn The turn to consider to update the list
     **/
    private void updateCoalitionList(Turn turn) {
        // for now we just consider that a coalition is a set of player
        // that have never attack directly each other
        // Construct coalition list        
        ArrayList l = new ArrayList();
        for(int i=0;i<game.getRemainingPlayer().size();i++) {
            Player p1 = (Player)game.getRemainingPlayer().get(i);
            ArrayList l2 = new ArrayList();
            l2.add((Player)p1.clone());
            //System.out.println("coal candidate avec "+p1.toString());
            for(int j=0;j<game.getRemainingPlayer().size();j++) {
                Player p2 = (Player)game.getRemainingPlayer().get(j);
                if (p2.equals(p1)) {
                    continue;
                }
                if (sameCoalition(p1,p2,turn.getPlayersTurnsList())) {
                    if (!(coalitionContains(l,l2))) {
                        //System.out.println("je peux m" +"mettre "+p2.toString()+" avec "+p1.toString());
                        l2.add((Player)p2.clone());
                    }
                }
            }
            if (simulations.SimulationParameters.debug) {
                System.out.println("candidat = "+l2.toString());
            }
            l.add(l2);
        }
        for(int i=0;i<l.size();i++) {
            ArrayList l2 = (ArrayList)l.get(i);
            if (l2.size()<=1) {
                l.remove(i);
                i--;
            }
        }
        coalitionList.clear();
        // l is the list all candidate coalition
        for(int i=0;i<l.size();i++) {
            ArrayList l2 = (ArrayList)l.get(i);
            if (isCoalition(l2,turn.getPlayersTurnsList())) {
                if (!(coalitionContains(coalitionList,l2))) {
                    if (simulations.SimulationParameters.debug) {
                        System.out.println("ajout definitif = "+l2.toString());
                    }
                    coalitionList.add(l2);
                }
            }
        }
        if (simulations.SimulationParameters.debug) {
            System.out.println("Coalition list = "+coalitionList.toString());
        }
        //if (coalitionList.size()>0) {
            coalitionListList.add((ArrayList)coalitionList.clone());
        //}
        // elt of coalitionRanking = list of 3 integer
            // first : nb times appears
            // second : lg max seq
            // third : nb seq
        Iterator it = ((ArrayList)coalitionList.clone()).iterator();
        while (it.hasNext()) {
            ArrayList coal = (ArrayList)it.next();
            //if (coalitionsRanking.containsKey(coal)) {
            ArrayList list = coalitionRankingContains(coal);
            
            if (list!=null) {
                //System.out.println("+1 sur "+list.toString());
                CoalitionData data = (CoalitionData)coalitionsRanking.get(list);
                data.setNbOcc(data.getNbOcc()+1);
                if (data.isLastTurn()) {
                    data.setCurrentLength(data.getCurrentLength()+1);
                    if (data.getMaxLength()<data.getCurrentLength()) {
                        data.setMaxLength(data.getCurrentLength());
                    }
                } else {
                    data.setSeqNbOcc(data.getSeqNbOcc()+1);
                    data.setLastTurn(true);
                    data.setCurrentLength(1);
                }
                coalitionsRanking.put(list,data);
            } else {
                //System.out.println("ajout "+coal.toString());
                //coalitionsRanking.put(coal,new Integer(1));
                coalitionsRanking.put(coal,new CoalitionData(1, 1, 1, 1, true));
            }
        }
        // 2nd pass
        it = coalitionsRanking.entrySet().iterator();
        while (it.hasNext()) {
            Entry coal = (Entry)it.next();
            if (!(coalitionContains(coalitionList, (ArrayList)coal.getKey()))) {
                CoalitionData data = (CoalitionData)coal.getValue();
                data.setLastTurn(false);
                coalitionsRanking.put(coal.getKey(), data);
            }
        }
        if (coalitionView!=null) {
            coalitionView.display(coalitionList,turn,turnDescription);
        }
    }
    
    private ArrayList coalitionRankingContains(ArrayList coal) {
        Iterator it = coalitionsRanking.entrySet().iterator();
        while (it.hasNext()) {
            Entry entry = (Entry)it.next();
            ArrayList l = (ArrayList)entry.getKey();
            if ((l.size()==coal.size())&&(coal.containsAll(l))) {
                return (ArrayList)entry.getKey();
            }
        }
        return null;
    }
    
    /** Returns the player's statistics of player
     * @param p The player
     * @return The the player's statistics of player
     **/
    private PlayersStats getPlayerStatistic(Player p) {
        Iterator it = statisticsList.iterator();
        while (it.hasNext()) {
            PlayersStats ps = (PlayersStats)it.next();
            if (ps.getPlayer().equals(p)) {
                return ps;
            }
        }
        return null;
    }
    
    /** Update the statistics of players considering last turn
     * @param turn The turn
     **/
    private void updateStatistics(Turn turn) {
        Iterator it = turn.getPlayersTurnsList().iterator();
        while (it.hasNext()) {
            PlayerTurn pt = (PlayerTurn)it.next();
            PlayersStats ps = getPlayerStatistic(pt.getPlayer());
            ps.update(pt);
        }
    }
    
    /** init the game : creates list instance, memorize time, etc... */
    public void initGame() {
            try {
                historyWriter = new HistoryWriter(game.getHistoryFile(),game.getTraceLevel());
            }catch(IOException e) {
                //System.out.println(e.toString());
                //System.out.println("Unable to save data of the game in file");
                historyWriter = null;
            }
        GregorianCalendar date = new GregorianCalendar();
        String s = date.get(Calendar.DAY_OF_MONTH)+"/"+(date.get(Calendar.MONTH)+1)+"/"+
        date.get(Calendar.YEAR)+" "+date.get(Calendar.HOUR_OF_DAY)+"h"+date.get(Calendar.MINUTE)+
        "m"+date.get(Calendar.SECOND)+"s"+date.get(Calendar.MILLISECOND)+"ms";
        s+=" - Begin of the game with "+game.getPlayerList().size()+" players\n";
        statisticsList.clear();
        Iterator it = game.getPlayerList().iterator();
        while (it.hasNext()) {
            Player player = (Player)it.next();
            s+=player.getName();
            s+=" - ";
            s+=player.getStrategy().toString();
            s+="\n";
            s+="Strategy description : \n";
            s+="Attack : \n";
            s+=player.getStrategy().getAttackComposition();
            s+="Asking : \n";
            s+=player.getStrategy().getAskingComposition();
            s+="Decision : \n";
            s+=player.getStrategy().getDecisionComposition();
            statisticsList.add(new PlayersStats(player));
        }
        try {
            if (historyWriter!=null) {
                historyWriter.write(s);
            }
        }catch(IOException e) {
            System.out.println("unable to write in history file");
        }
        gameHistory=new GameHistory();
        threeLastTurn = new ArrayList();
        classement = new ArrayList();
        coalitionListList = new ArrayList();
        coalitionsRanking = new Hashtable();
        allCoalitionsList.clear();
        //allCoalitionsList = getAllCoalitions();
        //System.out.println("allCoalitionList = "+allCoalitionsList);
        threeLastTurn.add(null);
        threeLastTurn.add(null);
        threeLastTurn.add(null);
        playerListList.clear();
        if (coalitionView!=null) {
            coalitionList.clear();
            coalitionList.add(game.getClonePlayerList());
        }
        addToGraph();
    }
    
    /** return true if game is finish because the three last turn are the same
     * false otherwise
     * @return return true if game is finish because the three last turn are the same
     * false otherwise
     **/
    public boolean nextGameTurn() {
        // d??cisions des joueurs
        // gameHistory.add(nextTurn());
        // insert last turn
        Turn turn = nextTurn();
        //System.out.println("------------next game TURN-------------");
        //turn.print();
        //System.out.println("------------Fin next game TURN-------------");
        if ((game.getPlayerList().size()>2)&&(game.isCoalitionSave())) {
            updateCoalitionList(turn);
        }
        //System.out.println("--------AP UPDATECOALITIONLIST-------------");
        try {
            if (!(game.isNoSave())) {
                historyWriter.write("Coalition list :\n");
                historyWriter.write(coalitionList.toString()+"\n");
            }
        }catch(IOException e) {
            System.out.println("Unable to write into the history file");
        }
        threeLastTurn.remove(0);
        //threeLastTurn.set(0,threeLastTurn.get(1));
        //threeLastTurn.set(1,threeLastTurn.get(2));
        //threeLastTurn.set(2,gameHistory.getTurnList().get(gameHistory.getTurnList().size()-1));
        threeLastTurn.add(turn);
        Iterator it2 = game.getRemainingPlayer().iterator();
        while (it2.hasNext()) {
            Player p = (Player)it2.next();
            //System.out.println("------------TURN-------------");
            //turn.print();
            //System.out.println("------------FIN TURN-------------");
            p.getStrategy().updateLastTurn(turn, (Player)p.clone(), game.getCloneRemainingPlayer());
        }
        if (!(game.isNoSave())) {
            updateStatistics(turn);
        }
        //gameHistory.add(turn);
        Turn turn1 = (Turn)threeLastTurn.get(0);
        Turn turn2 = (Turn)threeLastTurn.get(1);
        Turn turn3 = (Turn)threeLastTurn.get(2);
        /**if (turn1!=null) {
            System.out.println("TURN1");
            turn1.print();
            System.out.println("----------------------");
        }
        if (turn2!=null) {
            System.out.println("TURN2");
            turn2.print();
            System.out.println("----------------------");
        }
        if (turn3!=null) {
            System.out.println("TURN3");
            turn3.print();
            System.out.println("----------------------");
        }**/
        if (!((turn1==null)||(turn2==null)||(turn3==null))) {
            if ((turn1.equals(turn2))&&(turn2.equals(turn3))) {
                try {
                        historyWriter.write("Game finished with no winner because the three last turns are the same\n");
                }catch(IOException e) {
                    System.out.println("unable to write in history file");
                }
                return true;
            }
        }
        return false;
    }
    
    /** Returns a array of double that represents the data of l
     * @param l The list of coalition list
     * @return a array of double that represents the data of l
     **/
    public double[] coalitionListToData(ArrayList l) {
        double[] data = new double[l.size()];
        for(int i=0;i<l.size();i++) {
            data[i]=((ArrayList)l.get(i)).size();
        }
        return data;
    }
    
    /** end the game **/
    public void endGame() {
        Iterator it = game.getRemainingPlayer().iterator();
        while (it.hasNext()) {
            Player p = (Player)it.next();
            classement.add(0,new PlayerIntElement((Player)p.clone(),currentTurn));
        }
        try {
            historyWriter.write("Ranking : \n");
            it = classement.iterator();
            int cpt = 1;
            while (it.hasNext()) {
                PlayerIntElement elt = (PlayerIntElement)it.next();
                if (!(game.isNoSave())) {
                    historyWriter.write("rank #"+cpt+" : "+elt.getPlayer()+" "+elt.getPlayer().getStrategy()+" finshed on turn "+elt.getNb()+"\n");
                }
                System.out.println("rank #"+cpt+" : "+elt.getPlayer()+" "+elt.getPlayer().getStrategy()+" finshed on turn "+elt.getNb());
                cpt++;
            }
                historyWriter.close();
        }catch(IOException e) {
            System.out.println("unable to write in history file");
        }
        String sumFile = game.getHistoryFile().getPath();
        sumFile = sumFile.substring(0,sumFile.length()-4);
        sumFile+="_summary.txt";
        if (simulations.SimulationParameters.debug) {
            System.out.println("saving summary...");
        }
        new simulations.gui.GameSummarySaver(classement).save(new File(sumFile));
        if (simulations.SimulationParameters.debug) {
            System.out.println("saving done.");
        }
        if ((game.getGraphFile()!=null)&&(game.isNoSave()==false)) {
            if (simulations.SimulationParameters.debug) {
                System.out.println("saving graph...");
            }
            new simulations.gui.curves.CurvesDialog(null,false,classement,playerListList).save(game.getGraphFile());
            if (simulations.SimulationParameters.debug) {
                System.out.println("saving done.");
                System.out.println("saving coal graph...");
            }
            /**
            String name = game.getGraphFile().getPath().substring(0,game.getGraphFile().getPath().length()-4);
            name=name+"_coalition.png";
            new simulations.gui.curves.GenericCurvesDialog(null,false,coalitionListToData(coalitionListList),"").save(new File(name));
            if (simulations.SimulationParameters.debug) {
                System.out.println("saving done.");
            }**/
        }
        if (simulations.SimulationParameters.debug) {
            System.out.println("coalitionListList");
            it = coalitionListList.iterator();
            while (it.hasNext()) {
                System.out.println(it.next().toString());
            }
            System.out.println("coalition ranking");
            java.util.Set set = coalitionsRanking.entrySet();
            it = set.iterator();
            while (it.hasNext()) {
                System.out.println("-->"+(it.next()).toString());
            }
        }
        if (game.isCoalitionSave()) {
            String coalFile = game.getHistoryFile().getPath();
            coalFile = coalFile.substring(0,coalFile.length()-4);
            coalFile+="_coalition_ranking.html";
            if (simulations.SimulationParameters.debug) {
                System.out.println("saving coalition ranking...");
            }
            new simulations.gui.coalitions.CoalitionRankingSaver(coalitionsRanking,"").save(new File(coalFile));
            if (simulations.SimulationParameters.debug) {
                System.out.println("saving done.");
            }
        }
        String rankFile = game.getHistoryFile().getPath();
        rankFile = rankFile.substring(0,rankFile.length()-4);
        rankFile+="_players_ranking.html";
        if (simulations.SimulationParameters.debug) {
            System.out.println("saving players ranking...");
        }
        new simulations.gui.ClassementSaver(classement).save(new File(rankFile));
        if (simulations.SimulationParameters.debug) {
            System.out.println("saving done.");
            //System.out.println(coalitionsRanking.toString());
        }
    }
    
    /** launch the game
     * @return false if the game has stoppe because of the three last turn rule true otherwise
     **/
    public boolean playGame() {
        if (game==null) {
            return false;
        }
        initGame();
        boolean b = false;
        while ((!(game.isFinish()))&&(currentTurn<maxTurn)) {
            turnDescription=new String("");
            b = nextGameTurn();
            if (b) {
                break;
            }
        }
        endGame();
        return b;
    }
    
    /** This method returns the last turn of the game. The judge will use this method to
     * give informations about the last turn to players
     * @return The last turn of the game
     */
    public Turn returnLastTurn() {
        return ((Turn)gameHistory.getTurnList().get(gameHistory.getTurnList().size()-1)).getClone();
    }
    
    /** Returns (aclone) of the list of remaining players
     * @return Returns (a clone) of the list of remaining players
     */
    public ArrayList getCloneRemainingPlayers() {
        return game.getCloneRemainingPlayer();
    }
    
    /**
     * Add the last list of players to the playerListList to construct graph
     **/
    public void addToGraph() {
        // Construct a list of playerList
        playerListList.add((ArrayList)game.getClonePlayerList());
    }
    
    /** playing next turn of the game. Each player plays one shot.
     * @return Return the desrciption of the turn
     */
    public Turn nextTurn() {
        scoresUpdateToDo.clear();
        Iterator it = game.getPlayerList().iterator();
        while (it.hasNext()) {
            scoresUpdateToDo.add(new Score((Player)it.next(), 0));
        }
        currentTurn++;
        turnDescription="";
        if (simulations.SimulationParameters.debug) {
            System.out.println("turn #"+currentTurn);
        }
        try {
            if (!(game.isNoSave())) {
                historyWriter.write("---------------------------------------------------------\n");
                historyWriter.write("turn #"+currentTurn+"\n");
            }
        }catch(IOException e) {
            System.out.println("Unable to write into history file");
        }
        ArrayList playerTurnList = new ArrayList();
        ArrayList playerListClone = (ArrayList)game.getCloneRemainingPlayer().clone();
        ArrayList attackList = new ArrayList();
        //it = game.getRemainingPlayer().iterator();
        //System.out.println("Remaining players"+game.getRemainingPlayer().toString());
        for(int i=0;i<((ArrayList)game.getRemainingPlayer().clone()).size();i++) {
            //while (it.hasNext()) {
            //Player attackingPlayer = (Player)it.next();
            Player attackingPlayer = (Player)((ArrayList)game.getRemainingPlayer().clone()).get(i);
            Player playerToAttack = null;
            Player realPlayerToAttack = null;
            try {
                playerToAttack = attackingPlayer.getStrategy().getPlayerToAttack(
                (Player)attackingPlayer.clone(),(ArrayList)game.getCloneRemainingPlayer().clone());
                realPlayerToAttack=game.getPlayer(playerToAttack);
            }catch(NoPlayerToAttackException e) {
                System.out.println("turn #"+currentTurn+" / Player : "+attackingPlayer.toString()+e.toString());
                interrupt();
                System.exit(0);
            }catch(ManyPlayerToAttackException e) {
                System.out.println("turn #"+currentTurn+" / Player : "+attackingPlayer.toString()+e.toString());
                interrupt();
                System.exit(0);
            }
            if (!(game.getRemainingPlayer().contains(playerToAttack))) {
                Exception e = new AttackDeadPlayerException();
                System.out.println("turn #"+currentTurn+" / Player : "+attackingPlayer.toString()+e.toString());
                interrupt();
                System.exit(0);
            }
            if (playerToAttack==null) {
                Exception e = new NoPlayerToAttackException();
                System.out.println("turn #"+currentTurn+" / Player : "+attackingPlayer.toString()+e.toString());
                interrupt();
                System.exit(0);
            }
            if (playerToAttack.equals(attackingPlayer)) {
                Exception e = new AttackMySelfException();
                System.out.println("turn #"+currentTurn+" / Player : "+attackingPlayer.toString()+e.toString());
                interrupt();
                System.exit(0);
            }
            try {
                if (!(game.isNoSave())) {
                    historyWriter.write(attackingPlayer.toString()+" decided to attack "+playerToAttack.toString()+"\n");
                }
            }catch(IOException e) {
                System.out.println("Unable to write into history file");
            }
            turnDescription=turnDescription+attackingPlayer.toString()+" decided to attack "+playerToAttack.toString()+"\n";
            ArrayList playersToAskSupport = new ArrayList();
            ArrayList realPlayersToAskSupport = new ArrayList();
            playersToAskSupport=attackingPlayer.getStrategy().getPlayersToAskSupport(
            (Player)attackingPlayer.clone(),(Player)playerToAttack.clone(),(ArrayList)game.getCloneRemainingPlayer().clone());
            // We make sure that the player don't ask support support to himself
            playersToAskSupport.remove(attackingPlayer);
            // We make sure that he don't ask support to a dead player
            for(int j=0;j<playersToAskSupport.size();j++) {
                try {
                    if (!(game.getRemainingPlayer().contains(playersToAskSupport.get(j)))) {
                        playersToAskSupport.remove(playersToAskSupport.get(j));
                    }
                }catch(IndexOutOfBoundsException e) {
                    continue;
                }
            }
            Iterator it2 = playersToAskSupport.iterator();
            while (it2.hasNext()) {
                realPlayersToAskSupport.add(game.getPlayer((Player)it2.next()));
            }
            try {
                if (game.getTraceLevel()>=1) {
                    if (!(game.isNoSave())) {
                        historyWriter.write(attackingPlayer.toString()+" decided to ask support to "+playersToAskSupport.toString()+"\n");
                    }
                }
            }catch(IOException e) {
                System.out.println("Unable to write into history file");
            }
            turnDescription=turnDescription+attackingPlayer.toString()+" decided to ask support to "+playersToAskSupport.toString()+"\n";
            attackList.add(new Attack(attackingPlayer, realPlayerToAttack, realPlayersToAskSupport));
        }
        it = game.getRemainingPlayer().iterator();
        while (it.hasNext()) {
            Player p = (Player)it.next();
            p.getStrategy().updateAttacksInfos((Player)p.clone(),(ArrayList)attackList.clone());
        }
        int cpt=0;
        while (cpt<game.getRemainingPlayer().size()) {
            playerTurnList.add(doAttack(((Attack)attackList.get(cpt)).getAttackingPlayer(),
            ((Attack)attackList.get(cpt)).getAttackedPlayer(),
            ((Attack)attackList.get(cpt)).getAskingList()));
            cpt++;
        }
        endTurn();
        return new Turn(playerTurnList);
    }
    
    /**
     * This method returns true if the player p is already dead
     * and he's already in classement
     * @param p The player
     * @return true if the player p is already dead
     * and he's already in classement false otherwise
     */
    public boolean isInClassement(Player p) {
        Iterator it = classement.iterator();
        while (it.hasNext()) {
            PlayerIntElement elt = (PlayerIntElement)it.next();
            if (elt.getPlayer().equals(p)) {
                return true;
            }
        }
        return false;
    }
    
    /** validate the end of a turn.
     * Update truth between players.
     * write results of turn into history file.
     */
    public void endTurn() {
        // write current status in file
        try {
            if (!(game.isNoSave())) {
                historyWriter.write("End of attack\n");
            }
        }catch(IOException e) {
            System.out.println("Unable to write into history file");
        }
        // update scores
        Iterator it = scoresUpdateToDo.iterator();
        while (it.hasNext()) {
            Score score = (Score)it.next();
            Player playerToUpdate = (Player)score.getPlayer();
            Iterator it2 = game.getPlayerList().iterator();
            while (it2.hasNext()) {
                Player player = (Player)it2.next();
                if (player.equals(playerToUpdate)) {
                    player.setHitPoints(player.getHitPoints()-score.getScore());
                    break;
                }
            }
        }
        it = game.getPlayerList().iterator();
        while (it.hasNext()) {
            Player player = (Player)it.next();
            if (player.getHitPoints()<=0) {
                // he's dead
                // is he already in the classement ?
                //System.out.println(player.getName()+" est mort");
                if (!(isInClassement(player))) {
                    // I had him
                    //System.out.println(player.getName()+" Premiere fois");
                    classement.add(0,new PlayerIntElement((Player)player.clone(),currentTurn));
                }
            }
        }
        //System.out.println("END TURN : classement : "+classement.toString());
        it = game.getPlayerList().iterator();
        while (it.hasNext()) {
            Player player = (Player)it.next();
            try {
                if (!(game.isNoSave())) {
                    historyWriter.write(player.toString()+" : "+player.getHitPoints()+" HP remaining\n");
                }
            }catch(IOException e) {
                System.out.println("Unable to write into history file");
            }
            // The judge send infos about last turn to player's strategies
        }
        // Add infos to construct graph
        addToGraph();
    }
    
    /** Do an attack when all parameters are set.
     * @param attackingPlayer The attacking player
     * @param playerToAttack The attacked player
     * @param playersToAskSupport The players to ask support.
     * @return The description of the player turn
     */
    public PlayerTurn doAttack(Player attackingPlayer, Player playerToAttack, ArrayList playersToAskSupport) {
        PlayerTurn playerTurn = new PlayerTurn();
        playerTurn.setAttackedPlayer((Player)playerToAttack.clone());
        playerTurn.setPlayer((Player)attackingPlayer.clone());
        Iterator it = playersToAskSupport.iterator();
        ArrayList playersSupportingAttack = new ArrayList();
        try {
            if (game.getTraceLevel()>=2) {
                if (!(game.isNoSave())) {
                    historyWriter.write("----------decision of support ---------------------------------\n");
                }
            }
        }catch(IOException e) {
            System.out.println("Unable to write into history file");
        }
        turnDescription=turnDescription+"----------decision of support ---------------------------------\n";
        float sum = attackingPlayer.getHitPoints();
        ArrayList playersRefusingSupport = new ArrayList();
        playersRefusingSupport.clear();
        playersSupportingAttack.clear();
        while (it.hasNext()) {
            Player player = (Player)it.next();
            // player will support attack?
            if (player.getStrategy().supportAttack((Player)player.clone(), (Player)attackingPlayer.clone(), (Player)playerToAttack.clone(), (ArrayList)game.getRemainingPlayer().clone())) {
                playersSupportingAttack.add((Player)player.clone());
                try {
                    if (game.getTraceLevel()>=2) {
                        if (!(game.isNoSave())) {
                            historyWriter.write(player.toString()+" accept to support the attack of "+
                            attackingPlayer.toString()+" against "+playerToAttack.toString()+"\n");
                        }
                    }
                    turnDescription=turnDescription+player.toString()+" accept to support the attack of "+
                    attackingPlayer.toString()+" against "+playerToAttack.toString()+"\n";
                    sum+=player.getHitPoints();
                }catch(IOException e) {
                    System.out.println("unable to write into history file");
                }
            } else {
                playersRefusingSupport.add((Player)player.clone());
                try {
                    if (game.getTraceLevel()>=2) {
                        if (!(game.isNoSave())) {
                            historyWriter.write(player.toString()+" refuse to support the attack of "+
                            attackingPlayer.toString()+" against "+playerToAttack.toString()+"\n");
                        }
                    }
                    turnDescription=turnDescription+player.toString()+" refuse to support the attack of "+
                    attackingPlayer.toString()+" against "+playerToAttack.toString()+"\n";
                }catch(IOException e) {
                    System.out.println("unable to write into history file");
                }
            }
        }
        playerTurn.setPlayersAcceptingSupport((ArrayList)playersSupportingAttack.clone());
        playerTurn.setPlayersRefusingSupport((ArrayList)playersRefusingSupport.clone());
        // write attack into file
        //System.out.println("-----------------------DOATTACK--------------------");
        //System.out.println("players supporting attack : "+playersSupportingAttack.toString());
        //System.out.println("players refusing attack : "+playersRefusingSupport.toString());
        boolean win=false;
        try {
            if (game.getTraceLevel()>=1) {
                if (!(game.isNoSave())) {
                    historyWriter.write("Power of the attack : "+sum+" - Power of defense : "+playerToAttack.getHitPoints()+"\n");
                }
            }
        }catch(IOException e) {
            return null;
        }
        turnDescription=turnDescription+"Power of the attack : "+sum+" - Power of defense : "+playerToAttack.getHitPoints()+"\n";
        if (sum>playerToAttack.getHitPoints()) {
            win=true;
            try {
                if (!(game.isNoSave())) {
                    historyWriter.write("Attack succeed!\n");
                }
            }catch(IOException e) {
                System.out.println("Unable to write into history file");
            }
            turnDescription=turnDescription+"Attack succeed!\n";
            playerTurn.setSuccess(true);
        } else {
            try {
                if (!(game.isNoSave())) {
                    historyWriter.write("Attack missed!\n");
                }
            }catch(IOException e) {
                System.out.println("Unable to write into history file");
            }
            turnDescription=turnDescription+"Attack missed!\n";
            playerTurn.setSuccess(false);
        }
        // insert into update to do list
        boolean alone=false;
        if (win) {
            it = scoresUpdateToDo.iterator();
            Player playerToUpdate = null;
            while (it.hasNext()) {
                Score score = (Score)it.next();
                if (score.getPlayer().equals(playerToAttack)) {
                    score.setScore(score.getScore()+(game.getLoosingPoints()));
                    try {
                        if (game.getTraceLevel()>=1) {
                            if (!(game.isNoSave())) {
                                historyWriter.write(playerToAttack.getName()+" loose "+game.getLoosingPoints()+" HP\n");
                            }
                        }
                    }catch(IOException e) {
                        System.out.println("Unable to write into history file");
                    }
                }
                if (score.getPlayer().equals(attackingPlayer)) {
                    // if nobody support attack, player wins all points
                    if (playersSupportingAttack.size()==0) {
                        score.setScore(score.getScore()-game.getLoosingPoints());
                        alone=true;
                        try {
                            if (game.getTraceLevel()>=1) {
                                if (!(game.isNoSave())) {
                                    historyWriter.write(attackingPlayer.getName()+" wins "+game.getLoosingPoints()+" HP\n");
                                }
                            }
                        }catch(IOException e) {
                            System.out.println("Unable to write into history file");
                        }
                    } else {
                        score.setScore(score.getScore()-(game.getLoosingPoints()*game.getWinnerPoints()));
                        try {
                            if (game.getTraceLevel()>=1) {
                                if (!(game.isNoSave())) {
                                    historyWriter.write(attackingPlayer.getName()+" wins "+(game.getLoosingPoints()*game.getWinnerPoints())+" HP\n");
                                }
                            }
                        }catch(IOException e) {
                            System.out.println("Unable to write into history file");
                        }
                    }
                }
            }
            // update scores for players support the attack
            if ((game.isProportional())&&(!alone)) {
                if (simulations.SimulationParameters.debug) {
                    System.out.println("Redistribution proportionnelle");
                }
                // calculate the sum of ressources of the helping players
                // The average means that he receive exactly the coalitionnal points
                // With this we can calculate de proportionnal retribution
                it = playersSupportingAttack.iterator();
                sum = 0;
                while (it.hasNext()) {
                    Player p = (Player)it.next();
                    sum+=p.getHitPoints();
                }
                float avg = sum/playersSupportingAttack.size();
                float redistributedPoints=(game.getLoosingPoints()*game.getCoalitionPoints())/playersSupportingAttack.size();
                // Now we know that a player with avg ressources receive redistributedPoints HP
                it = playersSupportingAttack.iterator();
                while (it.hasNext()) {
                    Player player = (Player)it.next();
                    Iterator it2 = scoresUpdateToDo.iterator();
                    while (it2.hasNext()) {
                        Score score = (Score)it2.next();
                        if (score.getPlayer().equals(player)) {
                            float winPts = ((redistributedPoints*player.getHitPoints())/avg);
                            score.setScore(score.getScore()-winPts);
                            try {
                                if (game.getTraceLevel()>=1) {
                                    if (!(game.isNoSave())) {
                                        historyWriter.write(score.getPlayer().getName()+" wins "+winPts+" HP\n");
                                    }
                                }
                            }catch(IOException e) {
                                System.out.println("Unable to write into history file");
                            }
                        }
                    }
                }
            } else if (!(alone)) {
                float redistributedPoints=(game.getLoosingPoints()*game.getCoalitionPoints())/playersSupportingAttack.size();
                it = playersSupportingAttack.iterator();
                while (it.hasNext()) {
                    Player player = (Player)it.next();
                    Iterator it2 = scoresUpdateToDo.iterator();
                    while (it2.hasNext()) {
                        Score score = (Score)it2.next();
                        if (score.getPlayer().equals(player)) {
                            score.setScore(score.getScore()-redistributedPoints);
                            try {
                                if (game.getTraceLevel()>=1) {
                                    if (!(game.isNoSave())) {
                                        historyWriter.write(score.getPlayer().getName()+" wins "+redistributedPoints+" HP\n");
                                    }
                                }
                            }catch(IOException e) {
                                System.out.println("Unable to write into history file");
                            }
                        }
                    }
                }
            }
        }
        //System.out.println("Fin doAttack()");
        //System.out.println("accepting : "+playerTurn.getPlayersAcceptingSupport().toString());
        //System.out.println("refusing : "+playerTurn.getPlayersRefusingSupport().toString());
        return playerTurn;
    }
    
    /** Getter for property game.
     * @return Value of property game.
     *
     */
    public simulations.core.Game getGame() {
        return game;
    }
    
    /** Setter for property game.
     * @param game New value of property game.
     *
     */
    public void setGame(simulations.core.Game game) {
        this.game = game;
        try {
            historyWriter = new HistoryWriter(game.getHistoryFile(),game.getTraceLevel());
        }catch(IOException e) {
            //System.out.println(e.toString());
            //System.out.println("Unable to save data of the game in file");
            historyWriter = null;
        }
        currentTurn=0;
    }
    
    /** Getter for property currentTurn.
     * @return Value of property currentTurn.
     *
     */
    public int getCurrentTurn() {
        return currentTurn;
    }
    
    /** Setter for property currentTurn.
     * @param currentTurn New value of property currentTurn.
     *
     */
    public void setCurrentTurn(int currentTurn) {
        this.currentTurn = currentTurn;
    }
    
    /** Getter for property historyWriter.
     * @return Value of property historyWriter.
     *
     */
    public simulations.core.history.HistoryWriter getHistoryWriter() {
        return historyWriter;
    }
    
    /** Setter for property historyWriter.
     * @param historyWriter New value of property historyWriter.
     *
     */
    public void setHistoryWriter(simulations.core.history.HistoryWriter historyWriter) {
        this.historyWriter = historyWriter;
    }
    
    /** Getter for property playerListList.
     * @return Value of property playerListList.
     *
     */
    public java.util.ArrayList getPlayerListList() {
        return playerListList;
    }
    
    /** Setter for property playerListList.
     * @param playerListList New value of property playerListList.
     *
     */
    public void setPlayerListList(java.util.ArrayList playerListList) {
        this.playerListList = playerListList;
    }
    
    /** This method is overiden from Thread. It launch the thread. */
    public void run() {
        initGame();
    }
    
    /** Getter for property coalitionView.
     * @return Value of property coalitionView.
     *
     */
    public simulations.gui.coalitions.CoalitionView getCoalitionView() {
        return coalitionView;
    }
    
    /** Setter for property coalitionView.
     * @param coalitionView New value of property coalitionView.
     *
     */
    public void setCoalitionView(simulations.gui.coalitions.CoalitionView coalitionView) {
        this.coalitionView = coalitionView;
    }
    
    /** Getter for property statisticsList.
     * @return Value of property statisticsList.
     *
     */
    public java.util.ArrayList getStatisticsList() {
        return statisticsList;
    }
    
    /** Setter for property statisticsList.
     * @param statisticsList New value of property statisticsList.
     *
     */
    public void setStatisticsList(java.util.ArrayList statisticsList) {
        this.statisticsList = statisticsList;
    }
    
    /** Getter for property turnDescription.
     * @return Value of property turnDescription.
     *
     */
    public java.lang.String getTurnDescription() {
        return turnDescription;
    }
    
    /** Setter for property turnDescription.
     * @param turnDescription New value of property turnDescription.
     *
     */
    public void setTurnDescription(java.lang.String turnDescription) {
        this.turnDescription = turnDescription;
    }
    
    /**
     * Getter for property classement.
     * @return Value of property classement.
     */
    public java.util.ArrayList getClassement() {
        return classement;
    }
    
    /**
     * Setter for property classement.
     * @param classement New value of property classement.
     */
    public void setClassement(java.util.ArrayList classement) {
        this.classement = classement;
    }
    
    /**
     * Getter for property coalitionListList.
     * @return Value of property coalitionListList.
     */
    public java.util.ArrayList getCoalitionListList() {
        return coalitionListList;
    }
    
    /**
     * Setter for property coalitionListList.
     * @param coalitionListList New value of property coalitionListList.
     */
    public void setCoalitionListList(java.util.ArrayList coalitionListList) {
        this.coalitionListList = coalitionListList;
    }
    
    /**
     * Getter for property coalitionsRanking.
     * @return Value of property coalitionsRanking.
     */
    public java.util.Hashtable getCoalitionsRanking() {
        return coalitionsRanking;
    }
    
    /**
     * Setter for property coalitionsRanking.
     * @param coalitionsRanking New value of property coalitionsRanking.
     */
    public void setCoalitionsRanking(java.util.Hashtable coalitionsRanking) {
        this.coalitionsRanking = coalitionsRanking;
    }
    
}
