/*
 * Strategy.java
 *
 * Created on April 17, 2004, 6:16 PM
 */

package simulations.core.strategy;

import java.util.Vector;
import java.util.Random;
import java.io.Serializable;

/** This class is an abstract description of a strategy. It just define a name and a
 * random.
 * @author tom
 */
public abstract class Strategy implements Serializable, Cloneable {
    
    /** The random generator */    
    protected Random randomGenerator;
    
    /** The strategy's name */    
    protected String name;
    
    /** The memory **/
    protected int memory;
    
    /** The abrevation of the strat **/
    protected String abr;
    
    /** Creates a new instance of Strategy */
    public Strategy() {
        randomGenerator = new Random();
        name="";
    }
    
    /** Return the description of the strategy
     * @return Return the description of the strategy
     */    
    public abstract String getDescription();
    
    /** Creates a new instance of Strategy
     * @param seed The seed for the random
     */    
    public Strategy(int seed) {
        randomGenerator = new Random(seed);
    }
    
    /** Getter for property randomGenerator.
     * @return Value of property randomGenerator.
     *
     */
    public java.util.Random getRandomGenerator() {
        return randomGenerator;
    }
    
    /** Setter for property randomGenerator.
     * @param randomGenerator New value of property randomGenerator.
     *
     */
    public void setRandomGenerator(java.util.Random randomGenerator) {
        this.randomGenerator = randomGenerator;
    }
    
    /** Getter for property name.
     * @return Value of property name.
     *
     */
    public java.lang.String getName() {
        return name;
    }
    
    /** Setter for property name.
     * @param name New value of property name.
     *
     */
    public void setName(java.lang.String name) {
        this.name = name;
    }
    
    /** Overload of toString() method
     * This method return the name of the strategy
     * @return return the name of the strategy
     */    
    public String toString() {
        return name;
    }
    
    /** Overload of equals method
     * @param o The object to compare
     * @return Return true if this is equals to o false otherwise
     */    
    public boolean equals(Object o) {
        if (o instanceof Strategy) {
            if (name.equals(((Strategy)o).getName())) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * Getter for property memory.
     * @return Value of property memory.
     */
    public int getMemory() {
        return memory;
    }
    
    /**
     * Setter for property memory.
     * @param memory New value of property memory.
     */
    public void setMemory(int memory) {
        this.memory = memory;
    }
    
    /**
     * Getter for property abr.
     * @return Value of property abr.
     */
    public java.lang.String getAbr() {
        return abr;
    }
    
    /**
     * Setter for property abr.
     * @param abr New value of property abr.
     */
    public void setAbr(java.lang.String abr) {
        this.abr = abr;
    }
    
}
