/*
 * StrategyLoader.java
 *
 * Created on April 9, 2004, 4:08 AM
 */

package simulations.core.strategy;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.jar.JarFile;
import java.io.File;

import simulations.core.strategy.attackStrategy.AttackStrategy;
import simulations.core.strategy.askingSupportStrategy.AskingStrategy;
import simulations.core.strategy.decisionSupportStrategy.DecisionSupportStrategy;

/**
 * This class is used to lookup available strategies of the game
 * @author  tom
 */
public class StrategyLoader {
    
    /** Creates a new instance of StrategyLoader */
    public StrategyLoader() {
    }
    
    /** Return true if the game is launch with the jar file else elsewhere
     * @param s The string path
     * @return Return true if the game is launch with the jar file else elsewhere
     **/
    private boolean isJar(String s) {
        for(int i=0;i<(s.length()-5);i++) {
            String str = s.substring(i,i+5);
            //System.out.println(str);
            if (str.equals(".jar!")) {
                return true;
            }
        }
        return false;
    }
    
    /** Return the list of the complete strategies
     * @return Return the list of the complete strategies
     **/
    public ArrayList getAllCompleteStrategy() {
        ArrayList l = new ArrayList();
        // chercher toutes les strategies dispo
        // = lister le paquetage strategy et recuperer les noms des fichiers
        // pour chaque fichier instancier avec le classLoader et new Instance
        java.net.URL url = getClass().getResource("/simulations/core/strategy/Strategy.class");
        //System.out.println(url.getPath());
        if (isJar(url.getPath())) {
            // loading from jar
            JarFile jarFile = null;
            try {
                jarFile = new JarFile(System.getProperty("java.class.path"));
            }catch(java.io.IOException e) {
                System.out.println(e.toString());
            }
            //System.out.println(jarFile.getName());
            Enumeration enu = jarFile.entries();
            //ArrayList entriesList = (ArrayList)enu;
            for (; enu.hasMoreElements() ;) {
                String elt = enu.nextElement().toString();
                //System.out.println("elt = "+elt);
                //elt=elt.replace('/', '.');
                //System.out.println("elt2 = "+elt);
                String[] s = elt.split("/");
                //System.out.println("length="+s.length);
                if (s.length<=2) {
                    //System.out.println("continue");
                    continue;
                }
                String begin = "";
                for(int i=0;i<(s.length-1);i++) {
                    begin = begin.concat(s[i]+".");
                }
                //System.out.println("begin = "+begin);
                //System.out.println("end = "+s[s.length-1]);
                String ext = s[s.length-1].substring(s[s.length-1].length()-5,s[s.length-1].length());
                //System.out.println("ext = "+ext);
                if (begin.equals("simulations.core.strategy.completeStrategy.")) {
                    if (ext.equals("class")) {
                        String className = begin.concat(s[s.length-1]);
                        className = className.substring(0,className.length()-6);
                        if (simulations.SimulationParameters.debug) {
                            System.out.println("Loading "+className);
                        }
                        StrategyClassLoader loader = new StrategyClassLoader();
                        Class cl=null;
                        try {
                            cl = loader.loadClass(className);
                        }catch(ClassNotFoundException e) {
                            System.out.println(e.toString());
                            System.exit(0);
                        }
                        Strategy strat = null;
                        try {
                            strat = (Strategy)cl.newInstance();
                        }catch(InstantiationException e) {
                            continue;
                        }catch(IllegalAccessException e) {
                            System.out.println(e.toString());
                            System.exit(0);
                        }
                        if (!(strat.toString().equals("GenericComposedStrategy"))) {
                            l.add(strat);
                        }
                    }
                }    
            }
        } else {
            String[] s = url.getPath().split("/");
            String path="";
            for(int i=0;i<(s.length-1);i++) {
                path=path.concat(s[i]+"/");
            }
            //System.out.println(path);
            File f = new File(path+"completeStrategy/.");
            //System.out.println(f.getAbsolutePath());
            // si contient .jar!
            // tentative jarfile
            
            String[] list = f.list();
            for(int i=0;i<list.length;i++) {
                String str = list[i];
                //System.out.println(list[i]);
                String ext = str.substring(str.length()-6,str.length());
                //System.out.println(ext);
                if (ext.equals(".class")) {
                    //System.out.println("C'est un class");
                    String file = str.substring(0,str.length()-6);
                    //System.out.println(file);
                    String completeClass = "simulations.core.strategy.completeStrategy."+file;
                    if (simulations.SimulationParameters.debug) {
                        System.out.println("Loading "+completeClass);
                    }
                    StrategyClassLoader loader = new StrategyClassLoader();
                    Class cl=null;
                    try {
                        cl = loader.loadClass(completeClass);
                    }catch(ClassNotFoundException e) {
                        System.out.println(e.toString());
                        System.exit(0);
                    }
                    Strategy strat = null;
                    try {
                        strat = (Strategy)cl.newInstance();
                    }catch(InstantiationException e) {
                        continue;
                    }catch(IllegalAccessException e) {
                        System.out.println(e.toString());
                        System.exit(0);
                    }
                    if (!(strat.toString().equals("GenericComposedStrategy"))) {
                        l.add(strat);
                    }
                }
            }
        }
        return l;
    }
    
    /** Return the list of the attack strategies
     * @return Return the list of the attack strategies
     **/
    public ArrayList getAllAttackStrategy() {
        ArrayList l = new ArrayList();
        // chercher toutes les strategies dispo
        // = lister le paquetage strategy et recuperer les noms des fichiers
        // pour chaque fichier instancier avec le classLoader et new Instance
        java.net.URL url = getClass().getResource("/simulations/core/strategy/Strategy.class");
        //System.out.println(url.getPath());
        if (isJar(url.getPath())) {
            // loading from jar
            JarFile jarFile = null;
            try {
                jarFile = new JarFile(System.getProperty("java.class.path"));
            }catch(java.io.IOException e) {
                System.out.println(e.toString());
            }
            //System.out.println(jarFile.getName());
            Enumeration enu = jarFile.entries();
            //ArrayList entriesList = (ArrayList)enum;
            for (; enu.hasMoreElements() ;) {
                String elt = enu.nextElement().toString();
                //System.out.println("elt = "+elt);
                //elt=elt.replace('/', '.');
                //System.out.println("elt2 = "+elt);
                String[] s = elt.split("/");
                //System.out.println("length="+s.length);
                if (s.length<=2) {
                    //System.out.println("continue");
                    continue;
                }
                String begin = "";
                for(int i=0;i<(s.length-1);i++) {
                    begin = begin.concat(s[i]+".");
                }
                //System.out.println("begin = "+begin);
                //System.out.println("end = "+s[s.length-1]);
                String ext = s[s.length-1].substring(s[s.length-1].length()-5,s[s.length-1].length());
                //System.out.println("ext = "+ext);
                if (begin.equals("simulations.core.strategy.attackStrategy.")) {
                    if (ext.equals("class")) {
                        String className = begin.concat(s[s.length-1]);
                        className = className.substring(0,className.length()-6);
                        if (simulations.SimulationParameters.debug) {
                            System.out.println("Loading "+className);
                        }
                        StrategyClassLoader loader = new StrategyClassLoader();
                        Class cl=null;
                        try {
                            cl = loader.loadClass(className);
                        }catch(ClassNotFoundException e) {
                            System.out.println(e.toString());
                            System.exit(0);
                        }
                        AttackStrategy strat = null;
                        try {
                            strat = (AttackStrategy)cl.newInstance();
                        }catch(InstantiationException e) {
                            continue;
                        }catch(IllegalAccessException e) {
                            System.out.println(e.toString());
                            System.exit(0);
                        }
                        l.add(strat);
                    }
                }    
            }
        } else {
            String[] s = url.getPath().split("/");
            String path="";
            for(int i=0;i<(s.length-1);i++) {
                path=path.concat(s[i]+"/");
            }
            //System.out.println(path);
            File f = new File(path+"attackStrategy/.");
            //System.out.println(f.getAbsolutePath());
            // si contient .jar!
            // tentative jarfile
            
            String[] list = f.list();
            for(int i=0;i<list.length;i++) {
                String str = list[i];
                //System.out.println(list[i]);
                String ext = str.substring(str.length()-6,str.length());
                //System.out.println(ext);
                if (ext.equals(".class")) {
                    //System.out.println("C'est un class");
                    String file = str.substring(0,str.length()-6);
                    //System.out.println(file);
                    String completeClass = "simulations.core.strategy.attackStrategy."+file;
                    if (simulations.SimulationParameters.debug) {
                        System.out.println("Loading "+completeClass);
                    }
                    StrategyClassLoader loader = new StrategyClassLoader();
                    Class cl=null;
                    try {
                        cl = loader.loadClass(completeClass);
                    }catch(ClassNotFoundException e) {
                        System.out.println(e.toString());
                        System.exit(0);
                    }
                    AttackStrategy strat = null;
                    try {
                        strat = (AttackStrategy)cl.newInstance();
                    }catch(InstantiationException e) {
                        continue;
                    }catch(IllegalAccessException e) {
                        System.out.println(e.toString());
                        System.exit(0);
                    }
                    l.add(strat);
                }
            }
        }
        return l;
    }
    
    /** Return the list of the asking strategies
     * @return Return the list of the asking strategies
     **/
    public ArrayList getAllAskingStrategy() {
        ArrayList l = new ArrayList();
        // chercher toutes les strategies dispo
        // = lister le paquetage strategy et recuperer les noms des fichiers
        // pour chaque fichier instancier avec le classLoader et new Instance
        java.net.URL url = getClass().getResource("/simulations/core/strategy/Strategy.class");
        //System.out.println(url.getPath());
        if (isJar(url.getPath())) {
            // loading from jar
            JarFile jarFile = null;
            try {
                jarFile = new JarFile(System.getProperty("java.class.path"));
            }catch(java.io.IOException e) {
                System.out.println(e.toString());
            }
            //System.out.println(jarFile.getName());
            Enumeration enu = jarFile.entries();
            //ArrayList entriesList = (ArrayList)enu;
            for (; enu.hasMoreElements() ;) {
                String elt = enu.nextElement().toString();
                //System.out.println("elt = "+elt);
                //elt=elt.replace('/', '.');
                //System.out.println("elt2 = "+elt);
                String[] s = elt.split("/");
                //System.out.println("length="+s.length);
                if (s.length<=2) {
                    //System.out.println("continue");
                    continue;
                }
                String begin = "";
                for(int i=0;i<(s.length-1);i++) {
                    begin = begin.concat(s[i]+".");
                }
                //System.out.println("begin = "+begin);
                //System.out.println("end = "+s[s.length-1]);
                String ext = s[s.length-1].substring(s[s.length-1].length()-5,s[s.length-1].length());
                //System.out.println("ext = "+ext);
                if (begin.equals("simulations.core.strategy.askingSupportStrategy.")) {
                    if (ext.equals("class")) {
                        String className = begin.concat(s[s.length-1]);
                        className = className.substring(0,className.length()-6);
                        if (simulations.SimulationParameters.debug) {
                            System.out.println("Loading "+className);
                        }
                        StrategyClassLoader loader = new StrategyClassLoader();
                        Class cl=null;
                        try {
                            cl = loader.loadClass(className);
                        }catch(ClassNotFoundException e) {
                            System.out.println(e.toString());
                            System.exit(0);
                        }
                        AskingStrategy strat = null;
                        try {
                            strat = (AskingStrategy)cl.newInstance();
                        }catch(InstantiationException e) {
                            continue;
                        }catch(IllegalAccessException e) {
                            System.out.println(e.toString());
                            System.exit(0);
                        }
                        l.add(strat);
                    }
                }    
            }
        } else {
            String[] s = url.getPath().split("/");
            String path="";
            for(int i=0;i<(s.length-1);i++) {
                path=path.concat(s[i]+"/");
            }
            //System.out.println(path);
            File f = new File(path+"askingSupportStrategy/.");
            //System.out.println(f.getAbsolutePath());
            // si contient .jar!
            // tentative jarfile
            
            String[] list = f.list();
            for(int i=0;i<list.length;i++) {
                String str = list[i];
                //System.out.println(list[i]);
                String ext = str.substring(str.length()-6,str.length());
                //System.out.println(ext);
                if (ext.equals(".class")) {
                    //System.out.println("C'est un class");
                    String file = str.substring(0,str.length()-6);
                    //System.out.println(file);
                    String completeClass = "simulations.core.strategy.askingSupportStrategy."+file;
                    if (simulations.SimulationParameters.debug) {
                        System.out.println("Loading "+completeClass);
                    }
                    StrategyClassLoader loader = new StrategyClassLoader();
                    Class cl=null;
                    try {
                        cl = loader.loadClass(completeClass);
                    }catch(ClassNotFoundException e) {
                        System.out.println(e.toString());
                        System.exit(0);
                    }
                    AskingStrategy strat = null;
                    try {
                        strat = (AskingStrategy)cl.newInstance();
                    }catch(InstantiationException e) {
                        continue;
                    }catch(IllegalAccessException e) {
                        System.out.println(e.toString());
                        System.exit(0);
                    }
                    l.add(strat);
                }
            }
        }
        return l;
    }
    
    /** Return the list of the decision strategies
     * @return Return the list of the decision strategies
     **/
    public ArrayList getAllDecisionStrategy() {
        ArrayList l = new ArrayList();
        // chercher toutes les strategies dispo
        // = lister le paquetage strategy et recuperer les noms des fichiers
        // pour chaque fichier instancier avec le classLoader et new Instance
        java.net.URL url = getClass().getResource("/simulations/core/strategy/Strategy.class");
        //System.out.println(url.getPath());
        if (isJar(url.getPath())) {
            // loading from jar
            JarFile jarFile = null;
            try {
                jarFile = new JarFile(System.getProperty("java.class.path"));
            }catch(java.io.IOException e) {
                System.out.println(e.toString());
            }
            //System.out.println(jarFile.getName());
            Enumeration enu = jarFile.entries();
            //ArrayList entriesList = (ArrayList)enu;
            for (; enu.hasMoreElements() ;) {
                String elt = enu.nextElement().toString();
                //System.out.println("elt = "+elt);
                //elt=elt.replace('/', '.');
                //System.out.println("elt2 = "+elt);
                String[] s = elt.split("/");
                //System.out.println("length="+s.length);
                if (s.length<=2) {
                    //System.out.println("continue");
                    continue;
                }
                String begin = "";
                for(int i=0;i<(s.length-1);i++) {
                    begin = begin.concat(s[i]+".");
                }
                //System.out.println("begin = "+begin);
                //System.out.println("end = "+s[s.length-1]);
                String ext = s[s.length-1].substring(s[s.length-1].length()-5,s[s.length-1].length());
                //System.out.println("ext = "+ext);
                if (begin.equals("simulations.core.strategy.decisionSupportStrategy.")) {
                    if (ext.equals("class")) {
                        String className = begin.concat(s[s.length-1]);
                        className = className.substring(0,className.length()-6);
                        if (simulations.SimulationParameters.debug) {
                            System.out.println("Loading "+className);
                        }
                        StrategyClassLoader loader = new StrategyClassLoader();
                        Class cl=null;
                        try {
                            cl = loader.loadClass(className);
                        }catch(ClassNotFoundException e) {
                            System.out.println(e.toString());
                            System.exit(0);
                        }
                        DecisionSupportStrategy strat = null;
                        try {
                            strat = (DecisionSupportStrategy)cl.newInstance();
                        }catch(InstantiationException e) {
                            continue;
                        }catch(IllegalAccessException e) {
                            System.out.println(e.toString());
                            System.exit(0);
                        }
                        l.add(strat);
                    }
                }    
            }
        } else {
            String[] s = url.getPath().split("/");
            String path="";
            for(int i=0;i<(s.length-1);i++) {
                path=path.concat(s[i]+"/");
            }
            //System.out.println(path);
            File f = new File(path+"decisionSupportStrategy/.");
            //System.out.println(f.getAbsolutePath());
            // si contient .jar!
            // tentative jarfile
            
            String[] list = f.list();
            for(int i=0;i<list.length;i++) {
                String str = list[i];
                //System.out.println(list[i]);
                String ext = str.substring(str.length()-6,str.length());
                //System.out.println(ext);
                if (ext.equals(".class")) {
                    //System.out.println("C'est un class");
                    String file = str.substring(0,str.length()-6);
                    //System.out.println(file);
                    String completeClass = "simulations.core.strategy.decisionSupportStrategy."+file;
                    if (simulations.SimulationParameters.debug) {
                        System.out.println("Loading "+completeClass);
                    }
                    StrategyClassLoader loader = new StrategyClassLoader();
                    Class cl=null;
                    try {
                        cl = loader.loadClass(completeClass);
                    }catch(ClassNotFoundException e) {
                        System.out.println(e.toString());
                        System.exit(0);
                    }
                    DecisionSupportStrategy strat = null;
                    try {
                        strat = (DecisionSupportStrategy)cl.newInstance();
                    }catch(InstantiationException e) {
                        continue;
                    }catch(IllegalAccessException e) {
                        System.out.println(e.toString());
                        System.exit(0);
                    }
                    l.add(strat);
                }
            }
        }
        return l;
    }
    
}
