/*
 * AskingPlayersDontAttackMeStrategy2.java
 *
 * Created on April 23, 2004, 1:42 AM
 */

package simulations.core.strategy.askingSupportStrategy;

import simulations.core.Player;
import simulations.core.strategy.PlayerIntElement;
import java.util.ArrayList;
import java.util.Iterator;
import simulations.core.history.Turn;
import simulations.core.history.PlayerTurn;

/**
 * This class describes an asking strategy that ask to all players
 * that have never attack directly me and indirectly
 * @author  tom
 */
public class AskingPlayersDontAttackMeStrategy2 extends AskingStrategy {
    
    /** This list store the players that have never attack me directly
     * or indirecly **/
    private ArrayList notAskingList;
    
    /** True if its the first called of this strategy **/
    private boolean first;
    
    /** Creates a new instance of AskingPlayersDontAttackMeStrategy */
    public AskingPlayersDontAttackMeStrategy2() {
        super();
        name="AskingPlayersDontAttackMeStrategy2";
        notAskingList=new ArrayList();
        first=true;
        memoryParam=true;
        abr="4";
    }
    
    /** This method returns the description of the strategy
     * @return returns the description of the strategy
     */
    public String getDescription() {
        if (negative) {
            return "Asking support strategy :\n"+
            "I ask to all players that have never\n"+
            "attacked me directly and indirecly\n"+
            "memory="+memory+" COMPLEMENTARY\n";
        } else {
            return "Asking support strategy :\n"+
            "I ask to all players that have never\n"+
            "attacked me directly and indirecly\n"+
            "memory="+memory+"\n";
        }
    }
    
    /** This method returns true if the player p is already in notAskingList
     * @return true if the player p is already in notAskingList false otherwise
     **/
    private boolean containsPlayer(Player p) {
        for(int i=0;i<notAskingList.size();i++) {
            if (((PlayerIntElement)notAskingList.get(i)).getPlayer().equals(p)) {
                return true;
            }
        }
        return false;
    }
    
    /** This method put the nb of p at memory in the list
     * @param p The player to update
     **/
    private void updatePlayerMemory(Player p) {
        for(int i=0;i<notAskingList.size();i++) {
            PlayerIntElement elt = (PlayerIntElement)notAskingList.get(i);
            if (elt.getPlayer().equals(p)) {
                elt.setNb(memory);
                break;
            }
        }
    }
    
    /** Returns the list of players to ask support.
     * @return Returns the list of players to ask support.
     * @param attackedPlayer The attacked player
     * @param player The attacking player
     * @param playerList The list of <B>avalaible</B> player
     */
    public java.util.ArrayList getPlayersToAskSupport(simulations.core.Player player, simulations.core.Player attackedPlayer, java.util.ArrayList playerList) {
        if (simulations.SimulationParameters.debug) {
            System.out.println("notAskingList = "+notAskingList);
        }
        ArrayList l = new ArrayList();
        if (negative) {
            Iterator it = playerList.iterator();
            while (it.hasNext()) {
                Player p = (Player)it.next();
                if (containsPlayer(p)) {
                    l.add(p);
                }
            }
        } else {
            Iterator it = playerList.iterator();
            while (it.hasNext()) {
                Player p = (Player)it.next();
                if (!(containsPlayer(p))) {
                    l.add(p);
                }
            }
        }
        if (simulations.SimulationParameters.debug) {
            System.out.println("l = "+l);
        }
        return l;
    }
    
    /** This method is called by the judge to inform players of the last turn
     * @param turn The turn
     * @param player The owner of the strategy
     * @param remainingPlayers The list of the remaining players
     */
    public void updateLastTurn(simulations.core.history.Turn turn, simulations.core.Player player, java.util.ArrayList remainingPlayers) {
        // remove dead players
        for(int i=0;i<notAskingList.size();i++) {
            PlayerIntElement elt = (PlayerIntElement)notAskingList.get(i);
            elt.setNb(elt.getNb()-1);
        }
        Iterator it = turn.getPlayersTurnsList().iterator();
        while (it.hasNext()) {
            PlayerTurn pt = (PlayerTurn)it.next();
            if (pt.getAttackedPlayer().equals(player)) {
                // If he attack me I add him in my black list
                if (containsPlayer(pt.getPlayer())) {
                    updatePlayerMemory(pt.getPlayer());
                } else {
                    notAskingList.add(new PlayerIntElement(pt.getPlayer(),memory));
                }
                Iterator it2 = pt.getPlayersAcceptingSupport().iterator();
                while (it2.hasNext()) {
                    Player p = (Player)it2.next();
                    // remove player support an attack on me
                    if (containsPlayer(p)) {
                        updatePlayerMemory(p);
                    } else {
                        notAskingList.add(new PlayerIntElement(p,memory));
                    }
                }
            }
        }
        for(int i=0;i<notAskingList.size();i++) {
            PlayerIntElement elt = null;
            try {
                elt = (PlayerIntElement)notAskingList.get(i);
            }catch(IndexOutOfBoundsException e) {
                continue;
            }
            // memory turns have passed so I forget
            if (elt.getNb()==0) {
                notAskingList.remove(i);
                i--;
                continue;
            }
            if (!(remainingPlayers.contains(elt.getPlayer()))) {
                // This player no longer in remaining players list so I remove him
                notAskingList.remove(i);
                i--;
                continue;
            }
        }
    }
    
}
