/*
 * AskingStrategy.java
 *
 * Created on April 17, 2004, 7:38 PM
 */

package simulations.core.strategy.askingSupportStrategy;

import java.util.ArrayList;
import simulations.core.Player;
import simulations.core.history.Turn;
import java.io.Serializable;

/**
 * This abstract class describes haw to construct an asking strategy
 * @author  tom
 */
public abstract class AskingStrategy extends simulations.core.strategy.Strategy implements Cloneable, Serializable {
    
    /** True if the strategy is the negation **/
    protected boolean negative;
    
    /** True if the strategy is memory signifiant **/
    protected boolean memoryParam;
    
    /** Creates a new instance of AskingStrategy */
    public AskingStrategy() {
        super();
        negative=false;
    }
    
    /** Returns the list of players to ask support.
     * @return Returns the list of players to ask support.
     * @param attackedPlayer The attacked player
     * @param player The attacking player
     * @param playerList The list of <B>avalaible</B> player
     */    
    public abstract ArrayList getPlayersToAskSupport(Player player, Player attackedPlayer, ArrayList playerList);
    
    /** This method is called by the judge to inform players of the last turn
     * @param turn The turn
     * @param player The owner of the strategy
     * @param remainingPlayers The list of the remaining players
     */    
    public abstract void updateLastTurn(Turn turn, Player player, ArrayList remainingPlayers);
    
    /** This method returns the description of the strategy
     * @return returns the description of the strategy
     */    
    public abstract String getDescription();
    
    /** overload of equals method
     * @param o The object to compare
     * @return Returns true if this and o are equals
     **/
    public boolean equals(Object o) {
        if (o instanceof AskingStrategy) {
            if (name.equals(((AskingStrategy)o).getName())) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * Getter for property negative.
     * @return Value of property negative.
     */
    public boolean isNegative() {
        return negative;
    }
    
    /**
     * Setter for property negative.
     * @param negative New value of property negative.
     */
    public void setNegative(boolean negative) {
        this.negative = negative;
    }
    
    /**
     * Getter for property memoryParam.
     * @return Value of property memoryParam.
     */
    public boolean isMemoryParam() {
        return memoryParam;
    }
    
    /**
     * Setter for property memoryParam.
     * @param memoryParam New value of property memoryParam.
     */
    public void setMemoryParam(boolean memoryParam) {
        this.memoryParam = memoryParam;
    }
    
}
