/*
 * SpitefulAttackStrategy2.java
 *
 * Created on April 21, 2004, 2:08 AM
 */

package simulations.core.strategy.attackStrategy;

import simulations.core.Player;
import simulations.core.Attack;
import simulations.core.history.PlayerTurn;
import simulations.core.strategy.PlayerIntElement;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * This class describes a second spiteful strategy. It attack the player(s) 
 * that attack me directly or indirectly
 * @author  tom
 */
public class SpitefulAttackStrategy2 extends AttackStrategy {
    
    /** This list store the player that have already attack me or support an attack on me and are
     * still in the game **/
    private ArrayList attackList;
    
    /** Creates a new instance of SpitefulAttackStrategy2 */
    public SpitefulAttackStrategy2() {
        super();
        attackList = new ArrayList();
        name="SpitefulAttackStrategy2";
        memoryParam=true;
        abr="16";
    }
    
    /** return the description of this attack strategy
     * @return return the description of this attack strategy
     **/
    public String getDescription() {
        if (negative) {
            return "Attack strategy \n"+
            "I attack the players that have attacked me \n"+
            "or have support an attack on me\n"+
            "memory="+memory+" COMPLEMENTARY\n";
        } else {
            return "Attack strategy \n"+
            "I attack the players that have attacked me \n"+
            "or have support an attack on me\n"+
            "memory="+memory+"\n";
        }
    }
    
    /** return the player to attack
     * @param player The attacking player
     * @param playerList The list of <B>avalaible</B> player
     * @return The player to attack
     */    
    public java.util.ArrayList getPlayerToAttack(simulations.core.Player player, java.util.ArrayList playerList) {
        ArrayList l = new ArrayList();
        if (simulations.SimulationParameters.debug) {
            System.out.println("attackList = "+attackList);
        }
        Iterator it = attackList.iterator();
        while (it.hasNext()) {
            l.add(((PlayerIntElement)it.next()).getPlayer());
        }
        if (negative) {
            ArrayList l2 = new ArrayList();
            it = playerList.iterator();
            while (it.hasNext()) {
                Player p = (Player)it.next();
                if (p.equals(player)) {
                    continue;
                }
                if (!(l.contains(p))) {
                    l2.add(p);
                }
            }
            return l2;
        } else {
            return l;
        }
    }
    
    /** This method returns true if the player p is already in notAskingList
     * @return true if the player p is already in notAskingList false otherwise
     **/
    private boolean containsPlayer(Player p) {
        for(int i=0;i<attackList.size();i++) {
            if (((PlayerIntElement)attackList.get(i)).getPlayer().equals(p)) {
                return true;
            }
        }
        return false;
    }
    
    /** This method put the nb of p at memory in the list
     * @param p The player to update
     **/
    private void updatePlayerMemory(Player p) {
        for(int i=0;i<attackList.size();i++) {
            PlayerIntElement elt = (PlayerIntElement)attackList.get(i);
            if (elt.getPlayer().equals(p)) {
                elt.setNb(memory);
                break;
            }
        }
    }
    
    /** This method is called by the judge to inform players of the last turn
     * @param turn The turn
     * @param player The owner of the strategy
     * @param remainingPlayers The list of the remaining players
     */    
    public void updateLastTurn(simulations.core.history.Turn turn, simulations.core.Player player, java.util.ArrayList remainingPlayers) {
        if (simulations.SimulationParameters.debug) {
            System.out.println("UPDATE LAST TURN : ");
            turn.print();
            System.out.println("memory -1 sur elt");
        }
        for(int i=0;i<attackList.size();i++) {
            PlayerIntElement elt = (PlayerIntElement)attackList.get(i);
            elt.setNb(elt.getNb()-1);
        }
        Iterator it = turn.getPlayersTurnsList().iterator();
        while (it.hasNext()) {
            PlayerTurn pt = (PlayerTurn)it.next();
            if (pt.getAttackedPlayer().equals(player)) {
                // If he attack me I add him in my black list
                if (containsPlayer(pt.getPlayer())) {
                    if (simulations.SimulationParameters.debug) {
                        System.out.println("Update de "+pt.getPlayer());
                    }
                    updatePlayerMemory(pt.getPlayer());
                } else {
                    if (simulations.SimulationParameters.debug) {
                        System.out.println("Ajout de "+pt.getPlayer()+" il m'a attaque");
                    }
                    attackList.add(new PlayerIntElement(pt.getPlayer(),memory));
                }
                Iterator it2 = pt.getPlayersAcceptingSupport().iterator();
                while (it2.hasNext()) {
                    Player p = (Player)it2.next();
                    if (containsPlayer(p)) {
                        if (simulations.SimulationParameters.debug) {
                            System.out.println("Update de "+pt.getPlayer());
                        }
                        updatePlayerMemory(p);
                    } else {
                        if (simulations.SimulationParameters.debug) {
                            System.out.println("Ajout de "+pt.getPlayer()+" il a soutenu une attaque vs moi");
                        }
                        attackList.add(new PlayerIntElement(p,memory));
                    }
                }
            }
        }
        for(int i=0;i<attackList.size();i++) {
            PlayerIntElement elt = null;
            try {
                elt = (PlayerIntElement)attackList.get(i);
            }catch(IndexOutOfBoundsException e) {
                continue;
            }
            // memory turns have passed so I forget
            if (elt.getNb()==0) {
                if (simulations.SimulationParameters.debug) {
                    System.out.println("remove "+elt.getPlayer()+" pardon");
                }
                attackList.remove(i);
                i--;
                continue;
            }
            if (!(remainingPlayers.contains(elt.getPlayer()))) {
                // This player no longer in remaining players list so I remove him
                if (simulations.SimulationParameters.debug) {
                    System.out.println("remove "+elt.getPlayer().toString()+" mort");
                }
                attackList.remove(i);
                i--;
                continue;
            }
        }
    }
    
}
