/*
 * HistoryDialog.java
 *
 * Created on 29 janvier 2004, 23:16
 */

package simulations.gui;

import simulations.core.history.HistoryReader;

import java.io.IOException;

import javax.swing.SpinnerNumberModel;

/** This class describes the dialog to show the history of the game.
 * @author tom
 */
public class HistoryDialog extends javax.swing.JDialog {
    
    /** The reader of the history **/
    private HistoryReader historyReader;
    
    /** The current offset **/
    private int offset;
    
    /** Creates new form HistoryDialog
     * @param parent The parent frame
     * @param modal true if it's a modal dialog
     * @param historyReader The history reader for the history file
     */
    public HistoryDialog(java.awt.Frame parent, boolean modal, HistoryReader historyReader) {
        super(parent, modal);
        initComponents();
        this.historyReader=historyReader;
        historyReader.setOffset(0);
        //loadText();
        offset=-1;
        nextKoButtonActionPerformed(null);
        previousKoButton.setEnabled(false);
        pageSpinner.setModel(new SpinnerNumberModel(0,0,1000000,1));
        this.setSize(630, 420);
    }
    
    private void loadText() {
        try {
            String s = historyReader.read();
            if (simulations.SimulationParameters.debug) {
                System.out.println(s);
            }
            textPane.setPage("file://"+historyReader.getFile().getAbsolutePath());
        }catch(IOException e) {
            System.out.println("unable to read history");
        }
    }
    
    private void nextKo() {
        try {
            String s = (historyReader.nextKo());
            if (simulations.SimulationParameters.debug) {
                System.out.println(s);
            }
            textPane.setText(s);
        }catch(IOException e) {
            System.out.println("unable to read history");
        }
    }
    
    private void previousKo() {
        
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        titleLabel = new javax.swing.JLabel();
        buttonsPanel = new javax.swing.JPanel();
        previousKoButton = new javax.swing.JButton();
        nextKoButton = new javax.swing.JButton();
        closeButton = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        pageSpinner = new javax.swing.JSpinner();
        goButton = new javax.swing.JButton();
        scrollPane = new javax.swing.JScrollPane();
        textPane = new javax.swing.JTextPane();

        setTitle("History");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        titleLabel.setFont(new java.awt.Font("Dialog", 1, 16));
        titleLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        titleLabel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/edit.png")));
        titleLabel.setText("History of game");
        getContentPane().add(titleLabel, java.awt.BorderLayout.NORTH);

        previousKoButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/back.png")));
        previousKoButton.setText("Previous 100 lines");
        previousKoButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                previousKoButtonActionPerformed(evt);
            }
        });

        buttonsPanel.add(previousKoButton);

        nextKoButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/forward.png")));
        nextKoButton.setText("Next 100 lines");
        nextKoButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nextKoButtonActionPerformed(evt);
            }
        });

        buttonsPanel.add(nextKoButton);

        closeButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/button_cancel.png")));
        closeButton.setText("Close");
        closeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeButtonActionPerformed(evt);
            }
        });

        buttonsPanel.add(closeButton);

        jLabel1.setText("Go to page :");
        buttonsPanel.add(jLabel1);

        buttonsPanel.add(pageSpinner);

        goButton.setText("Go");
        goButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                goButtonActionPerformed(evt);
            }
        });

        buttonsPanel.add(goButton);

        getContentPane().add(buttonsPanel, java.awt.BorderLayout.SOUTH);

        textPane.setEditable(false);
        scrollPane.setViewportView(textPane);

        getContentPane().add(scrollPane, java.awt.BorderLayout.CENTER);

        pack();
    }//GEN-END:initComponents

    private void goButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_goButtonActionPerformed
        // TODO add your handling code here:
        int offset = ((Integer)pageSpinner.getValue()).intValue();
        if (offset<0) {
            return;
        }
        String s="";
        try {
            s=historyReader.next100LinesFrom(offset);
        }catch(IOException e) {
            
        }
        textPane.setText(s);
        if (offset==0) {
            previousKoButton.setEnabled(false);
        }
    }//GEN-LAST:event_goButtonActionPerformed

    private void previousKoButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_previousKoButtonActionPerformed
        // Add your handling code here:
        String s="";
        nextKoButton.setEnabled(true);
        offset--;
        try {
            s=historyReader.next100LinesFrom(offset);
        }catch(IOException e) {
            
        }
        textPane.setText(s);
        if (offset==0) {
            previousKoButton.setEnabled(false);
        }
    }//GEN-LAST:event_previousKoButtonActionPerformed

    private void nextKoButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nextKoButtonActionPerformed
        // Add your handling code here:
        previousKoButton.setEnabled(true);
        String s="";
        offset++;
        try {
            s=historyReader.next100LinesFrom(offset);
        }catch(IOException e) {
        }
        textPane.setText(s);
        if (s.equals("")) {
            nextKoButton.setEnabled(false);
        }
    }//GEN-LAST:event_nextKoButtonActionPerformed

    private void closeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeButtonActionPerformed
        // Add your handling code here:
        closeDialog(null);
    }//GEN-LAST:event_closeButtonActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel buttonsPanel;
    private javax.swing.JButton closeButton;
    private javax.swing.JButton goButton;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JButton nextKoButton;
    private javax.swing.JSpinner pageSpinner;
    private javax.swing.JButton previousKoButton;
    private javax.swing.JScrollPane scrollPane;
    private javax.swing.JTextPane textPane;
    private javax.swing.JLabel titleLabel;
    // End of variables declaration//GEN-END:variables
    
}
