/*
 * NewGameDialog.java
 *
 * Created on 19 janvier 2004, 23:23
 */

package simulations.gui;

import java.util.ArrayList;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Random;
import java.util.GregorianCalendar;
import java.util.Calendar;
import java.lang.Integer;
import java.lang.Long;
import java.lang.Float;
import java.lang.Double;
import java.lang.NumberFormatException;
import java.io.File;
import java.io.IOException;
import java.io.FileWriter;
import java.io.ObjectInputStream;
import java.io.FileInputStream;
import java.util.jar.JarFile;
import java.util.jar.JarEntry;

import javax.swing.JFileChooser;

import simulations.core.strategy.StrategyClassLoader;
import simulations.core.strategy.StrategyLoader;
import simulations.core.strategy.attackStrategy.AttackStrategy;
import simulations.core.strategy.askingSupportStrategy.AskingStrategy;
import simulations.core.strategy.decisionSupportStrategy.DecisionSupportStrategy;

import simulations.xml.XMLGameWriter;

import javax.swing.SpinnerNumberModel;
import javax.swing.filechooser.FileFilter;

import simulations.core.Player;
import simulations.core.strategy.Strategy;
import simulations.core.strategy.completeStrategy.*;

/** This class describes the dialog to enter parameters of a new game.
 * @author tom
 */
public class NewGameDialog extends javax.swing.JDialog {
    
    /** current step */
    private int step;
    
    /** List of players **/
    private ArrayList playerList;
    
    /** The random seed */
    private long seed;
    
    /** The point loosed by the attacked player **/
    private int loosingPoints;
    
    /** The % of loosingpoints won by the attackingPlayer **/
    private float winnerPoints;
    
    /** The % of loosingpoints won by the players supporting the attack **/
    private float coalitionPoints;
    
    /** True if the redistribution between the players supporting the attack are proportional by their resources **/
    private boolean proportional;
    
    /** The trace level for the history **/
    private int traceLevel;
    
    /** The list of all strategies **/
    private ArrayList strategyList;
    
    /** The graph file **/
    private File graphFile;
    
    /** The history file **/
    private File historyFile;
    
    /** The return status
     * 0 : Cancel click
     * 1 : Finish click
     **/
    private int returnStatus=0;
    
    /** The coalition definition **/
    private int coalitionDefinition;
    
    /** Creates new form NewGameDialog
     * @param parent The parent frame
     * @param modal true if it's a modal dialog
     */
    public NewGameDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
        init();
        step=1;
        tabbedPane.setEnabledAt(1,false);
        randomSeedSpinner.setModel(new SpinnerNumberModel(0,0,10000000,1));
        traceLevelSpinner.setModel(new SpinnerNumberModel(2,0,4,1));
        playersNumberSpinner.setModel(new SpinnerNumberModel(2,2,10000000,1));
        playerPointsSpinner.setModel(new SpinnerNumberModel(20.0,1.0,10000000.0,1.0));
        //coalitionalPointsSpinner.setModel(new SpinnerNumberModel(0.0,0.0,1.0,0.1));
        winnerPointsSpinner.setModel(new SpinnerNumberModel(1.0,0.0,1.0,0.1));
        this.setSize(730, 550);
        this.setLocationRelativeTo(parent);
    }
    
    private void refreshPlayerList() {
        
    }
    
    private boolean isJar(String s) {
        for(int i=0;i<(s.length()-16);i++) {
            String str = s.substring(i,i+16);
            if (simulations.SimulationParameters.debug) {
                System.out.println(str);
            }
            if (str.equals("simulations.jar!")) {
                return true;
            }
        }
        return false;
    }
    
    private void init() {
        playersNumberSpinner.setValue(new java.lang.Integer(2));
        strategyList = new StrategyLoader().getAllCompleteStrategy();
        strategyJList.setListData(strategyList.toArray());
        File f = new File(".");
        String s = f.getAbsolutePath().substring(0,f.getAbsolutePath().length()-1)+"history.txt";
        historyFile = new File(s);
        historyPathTextField.setText(s);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        buttonGroup1 = new javax.swing.ButtonGroup();
        TitleLabel = new javax.swing.JLabel();
        buttonsPanel = new javax.swing.JPanel();
        previousButton = new javax.swing.JButton();
        nextButton = new javax.swing.JButton();
        saveButton = new javax.swing.JButton();
        finishButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        tabbedPane = new javax.swing.JTabbedPane();
        firstPanel = new javax.swing.JPanel();
        labelPlayerNumber = new javax.swing.JLabel();
        playersNumberSpinner = new javax.swing.JSpinner();
        beginPointsTextField = new javax.swing.JTextField();
        beginPointsLabel = new javax.swing.JLabel();
        randomSeedLabel = new javax.swing.JLabel();
        randomSeedSpinner = new javax.swing.JSpinner();
        timeCheckBox = new javax.swing.JCheckBox();
        jLabel1 = new javax.swing.JLabel();
        loosingPointsTextField = new javax.swing.JTextField();
        winnerPointsSpinner = new javax.swing.JSpinner();
        labelPlayerNumber1 = new javax.swing.JLabel();
        labelPlayerNumber2 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        activatedCheckBox = new javax.swing.JCheckBox();
        coalitionPointsTextField = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        traceLevelSpinner = new javax.swing.JSpinner();
        jLabel6 = new javax.swing.JLabel();
        historyPathTextField = new javax.swing.JTextField();
        historyPathButton = new javax.swing.JButton();
        jLabel7 = new javax.swing.JLabel();
        coalitionDef1RadioButton = new javax.swing.JRadioButton();
        coalitionDef2RadioButton = new javax.swing.JRadioButton();
        coalitionDef3RadioButton = new javax.swing.JRadioButton();
        jLabel8 = new javax.swing.JLabel();
        graphPathTextField = new javax.swing.JTextField();
        graphPathButton = new javax.swing.JButton();
        coalitionDef4RadioButton = new javax.swing.JRadioButton();
        coalitionDef5RadioButton = new javax.swing.JRadioButton();
        strategyPanel = new javax.swing.JPanel();
        selectPlayerLabel = new javax.swing.JLabel();
        playerScrollPane = new javax.swing.JScrollPane();
        playerJList = new javax.swing.JList();
        selectStrategyLabel = new javax.swing.JLabel();
        strategyScrollPane = new javax.swing.JScrollPane();
        strategyJList = new javax.swing.JList();
        affectedStrategyLabel = new javax.swing.JLabel();
        affectedStrategyScrollPane = new javax.swing.JScrollPane();
        affectedStrategyJList = new javax.swing.JList();
        affectButton = new javax.swing.JButton();
        jLabel4 = new javax.swing.JLabel();
        playerPointsSpinner = new javax.swing.JSpinner();
        jLabel5 = new javax.swing.JLabel();
        playerNameTextField = new javax.swing.JTextField();
        jScrollPane1 = new javax.swing.JScrollPane();
        descriptionTextArea = new javax.swing.JTextArea();
        editButton = new javax.swing.JButton();
        LoadFromFileButton = new javax.swing.JButton();
        loadFromFolderButton = new javax.swing.JButton();

        setTitle("New Game");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        TitleLabel.setFont(new java.awt.Font("Dialog", 1, 16));
        TitleLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        TitleLabel.setText("New Game");
        getContentPane().add(TitleLabel, java.awt.BorderLayout.NORTH);

        previousButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/back.png")));
        previousButton.setText("Previous");
        previousButton.setToolTipText("Go to previous step");
        previousButton.setEnabled(false);
        previousButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                previousButtonActionPerformed(evt);
            }
        });

        buttonsPanel.add(previousButton);

        nextButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/forward.png")));
        nextButton.setText("Next");
        nextButton.setToolTipText("Go to next step");
        nextButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nextButtonActionPerformed(evt);
            }
        });

        buttonsPanel.add(nextButton);

        saveButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/save.png")));
        saveButton.setText("Save");
        saveButton.setEnabled(false);
        saveButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveButtonActionPerformed(evt);
            }
        });

        buttonsPanel.add(saveButton);

        finishButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/apply.png")));
        finishButton.setText("Finish");
        finishButton.setToolTipText("Validate this parameters");
        finishButton.setEnabled(false);
        finishButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                finishButtonActionPerformed(evt);
            }
        });

        buttonsPanel.add(finishButton);

        cancelButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/button_cancel.png")));
        cancelButton.setText("Cancel");
        cancelButton.setToolTipText("Close this window");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });

        buttonsPanel.add(cancelButton);

        getContentPane().add(buttonsPanel, java.awt.BorderLayout.SOUTH);

        tabbedPane.setTabPlacement(javax.swing.JTabbedPane.LEFT);
        firstPanel.setLayout(new java.awt.GridBagLayout());

        labelPlayerNumber.setText("Players number : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(labelPlayerNumber, gridBagConstraints);

        playersNumberSpinner.setToolTipText("spin to enter the number of players");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(playersNumberSpinner, gridBagConstraints);

        beginPointsTextField.setColumns(10);
        beginPointsTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        beginPointsTextField.setText("20");
        beginPointsTextField.setToolTipText("Enter here the number of hit points by player at the beginning of the game");
        beginPointsTextField.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                beginPointsTextFieldKeyTyped(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(beginPointsTextField, gridBagConstraints);

        beginPointsLabel.setText("Begin points : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(beginPointsLabel, gridBagConstraints);

        randomSeedLabel.setText("Random seed : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(randomSeedLabel, gridBagConstraints);

        randomSeedSpinner.setToolTipText("Spin to enter the seed of random numbers generation");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(randomSeedSpinner, gridBagConstraints);

        timeCheckBox.setText("Time");
        timeCheckBox.setToolTipText("Check this box to set the seed with a fonction of time");
        timeCheckBox.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                timeCheckBoxMouseClicked(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(timeCheckBox, gridBagConstraints);

        jLabel1.setText("Loosing points :");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(jLabel1, gridBagConstraints);

        loosingPointsTextField.setColumns(10);
        loosingPointsTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        loosingPointsTextField.setText("1");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(loosingPointsTextField, gridBagConstraints);

        winnerPointsSpinner.setToolTipText("spin to enter the number of players");
        winnerPointsSpinner.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                winnerPointsSpinnerStateChanged(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(winnerPointsSpinner, gridBagConstraints);

        labelPlayerNumber1.setText("Winner points :");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(labelPlayerNumber1, gridBagConstraints);

        labelPlayerNumber2.setText("Coalition points :");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(labelPlayerNumber2, gridBagConstraints);

        jLabel2.setText("Proportional side-payment :");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        firstPanel.add(jLabel2, gridBagConstraints);

        activatedCheckBox.setText("activated");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(activatedCheckBox, gridBagConstraints);

        coalitionPointsTextField.setColumns(10);
        coalitionPointsTextField.setEditable(false);
        coalitionPointsTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        coalitionPointsTextField.setText("0");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(coalitionPointsTextField, gridBagConstraints);

        jLabel3.setText("Trace level for history :");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(jLabel3, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(traceLevelSpinner, gridBagConstraints);

        jLabel6.setText("History file :");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 8;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(jLabel6, gridBagConstraints);

        historyPathTextField.setColumns(10);
        historyPathTextField.setEditable(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 8;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(historyPathTextField, gridBagConstraints);

        historyPathButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/folder.png")));
        historyPathButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        historyPathButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyPathButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 8;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(historyPathButton, gridBagConstraints);

        jLabel7.setText("Coalition definition :");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 10;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        firstPanel.add(jLabel7, gridBagConstraints);

        coalitionDef1RadioButton.setFont(new java.awt.Font("Dialog", 1, 10));
        coalitionDef1RadioButton.setSelected(true);
        coalitionDef1RadioButton.setText("1. don't attack directlty each others");
        buttonGroup1.add(coalitionDef1RadioButton);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 10;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        firstPanel.add(coalitionDef1RadioButton, gridBagConstraints);

        coalitionDef2RadioButton.setFont(new java.awt.Font("Dialog", 1, 10));
        coalitionDef2RadioButton.setText("2. don't attack directly and indirectly each others ");
        buttonGroup1.add(coalitionDef2RadioButton);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 11;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        firstPanel.add(coalitionDef2RadioButton, gridBagConstraints);

        coalitionDef3RadioButton.setFont(new java.awt.Font("Dialog", 1, 10));
        coalitionDef3RadioButton.setText("3. don't attack directly and indirectly each others and support their attacks");
        buttonGroup1.add(coalitionDef3RadioButton);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 12;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        firstPanel.add(coalitionDef3RadioButton, gridBagConstraints);

        jLabel8.setText("GraphFile : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 9;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(jLabel8, gridBagConstraints);

        graphPathTextField.setColumns(10);
        graphPathTextField.setEditable(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 9;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(graphPathTextField, gridBagConstraints);

        graphPathButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/folder.png")));
        graphPathButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        graphPathButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                graphPathButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 9;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(graphPathButton, gridBagConstraints);

        coalitionDef4RadioButton.setFont(new java.awt.Font("Dialog", 1, 10));
        coalitionDef4RadioButton.setText("4. 2 + attack the same player");
        buttonGroup1.add(coalitionDef4RadioButton);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 13;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(coalitionDef4RadioButton, gridBagConstraints);

        coalitionDef5RadioButton.setFont(new java.awt.Font("Dialog", 1, 10));
        coalitionDef5RadioButton.setText("5. 3 + attack the same player");
        buttonGroup1.add(coalitionDef5RadioButton);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 14;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        firstPanel.add(coalitionDef5RadioButton, gridBagConstraints);

        tabbedPane.addTab("Step #1", firstPanel);

        strategyPanel.setLayout(new java.awt.GridBagLayout());

        selectPlayerLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        selectPlayerLabel.setText("Select player");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        strategyPanel.add(selectPlayerLabel, gridBagConstraints);

        playerJList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        playerJList.setToolTipText("Select here the player tou want to affect strategy");
        playerJList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                playerJListValueChanged(evt);
            }
        });

        playerScrollPane.setViewportView(playerJList);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridheight = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        strategyPanel.add(playerScrollPane, gridBagConstraints);

        selectStrategyLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        selectStrategyLabel.setText("Select strategy");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        strategyPanel.add(selectStrategyLabel, gridBagConstraints);

        strategyJList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        strategyJList.setToolTipText("Select the strategy for the selected player");
        strategyJList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                strategyJListValueChanged(evt);
            }
        });

        strategyScrollPane.setViewportView(strategyJList);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridheight = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        strategyPanel.add(strategyScrollPane, gridBagConstraints);

        affectedStrategyLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        affectedStrategyLabel.setText("Affected strategy");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        strategyPanel.add(affectedStrategyLabel, gridBagConstraints);

        affectedStrategyJList.setToolTipText("The summary of affected strategy");
        affectedStrategyScrollPane.setViewportView(affectedStrategyJList);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 8;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        strategyPanel.add(affectedStrategyScrollPane, gridBagConstraints);

        affectButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/down.png")));
        affectButton.setText("Affect");
        affectButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                affectButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.gridwidth = 2;
        strategyPanel.add(affectButton, gridBagConstraints);

        jLabel4.setText("Points ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        strategyPanel.add(jLabel4, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        strategyPanel.add(playerPointsSpinner, gridBagConstraints);

        jLabel5.setText("Name ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        strategyPanel.add(jLabel5, gridBagConstraints);

        playerNameTextField.setColumns(8);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        strategyPanel.add(playerNameTextField, gridBagConstraints);

        jScrollPane1.setHorizontalScrollBarPolicy(javax.swing.JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        descriptionTextArea.setEditable(false);
        descriptionTextArea.setLineWrap(true);
        descriptionTextArea.setWrapStyleWord(true);
        jScrollPane1.setViewportView(descriptionTextArea);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        strategyPanel.add(jScrollPane1, gridBagConstraints);

        editButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/edit.png")));
        editButton.setText("Edit new strategy");
        editButton.setMargin(new java.awt.Insets(0, 14, 0, 14));
        editButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        strategyPanel.add(editButton, gridBagConstraints);

        LoadFromFileButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/folder.png")));
        LoadFromFileButton.setText("Add from file");
        LoadFromFileButton.setMargin(new java.awt.Insets(0, 14, 0, 14));
        LoadFromFileButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                LoadFromFileButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        strategyPanel.add(LoadFromFileButton, gridBagConstraints);

        loadFromFolderButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/folder.png")));
        loadFromFolderButton.setText("Load from folder");
        loadFromFolderButton.setMargin(new java.awt.Insets(0, 14, 0, 14));
        loadFromFolderButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadFromFolderButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        strategyPanel.add(loadFromFolderButton, gridBagConstraints);

        tabbedPane.addTab("Step #2", strategyPanel);

        getContentPane().add(tabbedPane, java.awt.BorderLayout.CENTER);

        pack();
    }//GEN-END:initComponents

    private void loadFromFolderButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadFromFolderButtonActionPerformed
        // TODO add your handling code here:
        JFileChooser chooser = new JFileChooser();
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            if (simulations.SimulationParameters.debug) {
                System.out.println("You chose to save this file: " +
                chooser.getSelectedFile().getName());
            }
            File f = new File(chooser.getSelectedFile().getPath());
            if (!(f.isDirectory())) {
                MsgBox msgBox = new MsgBox(null,true,f.getPath(),"is not a directory",false);
                msgBox.setVisible(true);
                return;
            }
            File[] list = f.listFiles();
            if (simulations.SimulationParameters.debug) {
                System.out.println(list.length+" files in "+f.getPath());
            }
            for(int i=0;i<list.length;i++) {
                File f2 = list[i];
                if (!(f2.isFile())) {
                    continue;
                }
                if (!(f2.getPath().substring(f2.getPath().length()-6,f2.getPath().length()).equals(".strat"))) {
                    continue;
                }
                if (simulations.SimulationParameters.debug) {
                    System.out.println("Loading from "+f2.getPath());
                }
                GenericComposedStrategy savedStrategy = new GenericComposedStrategy();
                try {
                    FileInputStream fos = new FileInputStream(f2);
                    ObjectInputStream oos = new ObjectInputStream(fos);
                    savedStrategy = (GenericComposedStrategy)oos.readObject();
                    oos.close();
                }catch(Exception e) {
                    System.out.println(e.toString());
                    MsgBox msgBox = new MsgBox(null, true, "Unable to load strategy","",false);
                    msgBox.setVisible(true);
                    return;
                }
                if (strategyList.contains(savedStrategy)) {
                    MsgBox msgBox = new MsgBox(null, true, "This strategy is already in the list", "", false);
                    msgBox.setVisible(true);
                    return;
                }
                if (simulations.SimulationParameters.debug) {
                    System.out.println("adding "+savedStrategy.getName());
                }
                strategyList.add(savedStrategy);
            }
            strategyJList.setListData(strategyList.toArray());
        }
    }//GEN-LAST:event_loadFromFolderButtonActionPerformed
    
    private void saveButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveButtonActionPerformed
        // TODO add your handling code here:
        javax.swing.JFileChooser chooser = new javax.swing.JFileChooser();//cr?ation dun nouveau filechosser
        chooser.setFileFilter(new StrategyFileFilter("xml"));
        //chooser.setApproveButtonText("Choose" ); //intitul? du bouton
        //chooser.showOpenDialog(null); //affiche la boite de dialogue
        if (chooser.showSaveDialog(null)== javax.swing.JFileChooser.APPROVE_OPTION) {
            if (simulations.SimulationParameters.debug) {
                System.out.println(chooser.getSelectedFile().getAbsolutePath());
            }
            String fileName = chooser.getSelectedFile().getPath();
            if (simulations.SimulationParameters.debug) {
                System.out.println("ext = "+fileName.substring(fileName.length()-4,fileName.length()));
            }
            if (!(fileName.substring(fileName.length()-4,fileName.length()).equals(".xml"))) {
                fileName=fileName+".xml";
            }
            File file = new File(fileName);
            if (simulations.SimulationParameters.debug) {
                System.out.println("XML file : "+file.getPath());
            }
            XMLGameWriter writer = new XMLGameWriter();
            writer.create(file, seed, loosingPoints, winnerPoints, coalitionPoints, traceLevel,
            coalitionDefinition, proportional, historyFile, graphFile, playerList);
        }
    }//GEN-LAST:event_saveButtonActionPerformed
    
    private void graphPathButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_graphPathButtonActionPerformed
        // TODO add your handling code here:
        JFileChooser chooser = new JFileChooser();
        StrategyFileFilter filter = new StrategyFileFilter("png");
        GenericComposedStrategy savedStrategy = new GenericComposedStrategy();
        filter.setDescription("PNG file");
        chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            if (simulations.SimulationParameters.debug) {
                System.out.println("You chose to save this file: " +
                chooser.getSelectedFile().getName());
            }
            String fileName = chooser.getSelectedFile().getPath();
            if (simulations.SimulationParameters.debug) {
                System.out.println(fileName);
            }
            if (!(fileName.substring(fileName.length()-4,fileName.length()).equals(".png"))) {
                fileName=fileName+".png";
            }
            graphFile = new File(fileName);
            graphPathTextField.setText(fileName);
        }
    }//GEN-LAST:event_graphPathButtonActionPerformed
    
    private void historyPathButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyPathButtonActionPerformed
        // Add your handling code here:
        JFileChooser chooser = new JFileChooser();
        StrategyFileFilter filter = new StrategyFileFilter("txt");
        GenericComposedStrategy savedStrategy = new GenericComposedStrategy();
        filter.setDescription("Text file");
        chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            if (simulations.SimulationParameters.debug) {
                System.out.println("You chose to save this file: " +
                chooser.getSelectedFile().getName());
            }
            String fileName = chooser.getSelectedFile().getPath();
            if (simulations.SimulationParameters.debug) {
                System.out.println(fileName);
            }
            if (!(fileName.substring(fileName.length()-4,fileName.length()).equals(".txt"))) {
                fileName=fileName+".txt";
            }
            historyFile = new File(fileName);
            historyPathTextField.setText(fileName);
        }
    }//GEN-LAST:event_historyPathButtonActionPerformed
    
    private void LoadFromFileButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_LoadFromFileButtonActionPerformed
        // Add your handling code here:
        JFileChooser chooser = new JFileChooser();
        StrategyFileFilter filter = new StrategyFileFilter("strat");
        GenericComposedStrategy savedStrategy = new GenericComposedStrategy();
        filter.setDescription("Strategy file");
        chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            if (simulations.SimulationParameters.debug) {
                System.out.println("You chose to save this file: " +
                chooser.getSelectedFile().getName());
            }
            String fileName = chooser.getSelectedFile().getPath();
            if (simulations.SimulationParameters.debug) {
                System.out.println(fileName);
            }
            try {
                File f = new File(fileName);
                FileInputStream fos = new FileInputStream(f);
                ObjectInputStream oos = new ObjectInputStream(fos);
                savedStrategy = (GenericComposedStrategy)oos.readObject();
                oos.close();
            }catch(Exception e) {
                System.out.println(e.toString());
                MsgBox msgBox = new MsgBox(null, true, "Unable to load strategy","",false);
                msgBox.setVisible(true);
                return;
            }
            if (strategyList.contains(savedStrategy)) {
                MsgBox msgBox = new MsgBox(null, true, "This strategy is already in the list", "", false);
                msgBox.setVisible(true);
                return;
            }
            strategyList.add(savedStrategy);
            strategyJList.setListData(strategyList.toArray());
        }
    }//GEN-LAST:event_LoadFromFileButtonActionPerformed
    
    private void editButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editButtonActionPerformed
        // Add your handling code here:
        ComposedStrategyDialog dialog = new ComposedStrategyDialog(null, true);
        dialog.setVisible(true);
        if (dialog.getReturnStatus()==1) {
            if (strategyList.contains(dialog.getEditedStrategy())) {
                MsgBox msgBox = new MsgBox(null, true, "This strategy is already in the list", "", false);
                msgBox.setVisible(true);
                return;
            }
            strategyList.add(dialog.getEditedStrategy());
            strategyJList.setListData(strategyList.toArray());
        }
    }//GEN-LAST:event_editButtonActionPerformed
    
    private void strategyJListValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_strategyJListValueChanged
        // Add your handling code here:
        if (strategyJList.getSelectedValue()!=null) {
            descriptionTextArea.setText(((Strategy)strategyJList.getSelectedValue()).getDescription());
        }
    }//GEN-LAST:event_strategyJListValueChanged
    
    private void playerJListValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_playerJListValueChanged
        // Add your handling code here:
        if (playerJList.getSelectedValue()==null) {
            return;
        }
        Player player = (Player)playerJList.getSelectedValue();
        playerNameTextField.setText(player.getName());
        playerPointsSpinner.setValue(new Double(player.getHitPoints()));
    }//GEN-LAST:event_playerJListValueChanged
    
    private void winnerPointsSpinnerStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_winnerPointsSpinnerStateChanged
        // Add your handling code here:
        if (simulations.SimulationParameters.debug) {
            System.out.println("handler State");
        }
        Double tmp = null;
        try {
            tmp = (Double)winnerPointsSpinner.getValue();
        }catch(NumberFormatException e) {
            System.out.println(e.toString());
            return;
        }
        //System.out.println(tmp.doubleValue());
        Float tmp2=new Float(tmp.floatValue());
        if (simulations.SimulationParameters.debug) {
            System.out.println(tmp.floatValue());
            System.out.println(tmp2.floatValue());
        }
        if (tmp2.equals(new Float(0.0))) {
            coalitionPointsTextField.setText("1.0");
        } else if (tmp2.equals(new Float(0.1))) {
            coalitionPointsTextField.setText("0.9");
        } else if (tmp2.equals(new Float(0.2))) {
            coalitionPointsTextField.setText("0.8");
        } else if (tmp2.equals(new Float(0.3))) {
            coalitionPointsTextField.setText("0.7");
        } else if (tmp2.equals(new Float(0.4))) {
            coalitionPointsTextField.setText("0.6");
        } else if (tmp2.equals(new Float(0.5))) {
            coalitionPointsTextField.setText("0.5");
        } else if (tmp2.equals(new Float(0.6))) {
            coalitionPointsTextField.setText("0.4");
        } else if (tmp2.equals(new Float(0.7))) {
            coalitionPointsTextField.setText("0.3");
        } else if (tmp2.equals(new Float(0.8))) {
            coalitionPointsTextField.setText("0.2");
        } else if (tmp2.equals(new Float(0.9))) {
            coalitionPointsTextField.setText("0.1");
        } else {
            coalitionPointsTextField.setText("0.0");
        }
    }//GEN-LAST:event_winnerPointsSpinnerStateChanged
    
    private boolean checkName(ArrayList l,String n) {
        Iterator it = l.iterator();
        while (it.hasNext()) {
            Player p = (Player)it.next();
            if (p.getName().equals(n)) {
                return false;
            }
        }
        return true;
    }
    
    private void affectButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_affectButtonActionPerformed
        // Add your handling code here:
        if (playerJList.getSelectedValue()==null) {
            return;
        }
        if (strategyJList.getSelectedValue()==null) {
            return;
        }
        Object[] players = playerJList.getSelectedValues();
        CompleteStrategy strat = null;
        CompleteStrategy strategy = null;
        try {
            strat = (CompleteStrategy)strategyJList.getSelectedValue();
            //strategy = (CompleteStrategy)strat.getClass().getConstructor(null).newInstance(null);
            
            /**StrategyClassLoader loader = new StrategyClassLoader();
             * strategy = (CompleteStrategy)loader.loadClass(strat.getClass().getName()).newInstance();
             * Vector attackList = new Vector();
             * Vector askingList = new Vector();
             * Vector decisionList = new Vector();**/
            
            strategy = (CompleteStrategy)strat.getClass().newInstance();
            strategy.setRandomGenerator(strat.getRandomGenerator());
            strategy.setName(strat.getName());
            strategy.setMemory(strat.getMemory());
            Iterator it = strat.getAttackStrategyVector().iterator();
            Vector attackList = new Vector();
            while (it.hasNext()) {
                AttackStrategy attackStrat = (AttackStrategy)it.next();
                AttackStrategy attackStrat2 = (AttackStrategy)attackStrat.getClass().newInstance();
                attackStrat2.setMemory(attackStrat.getMemory());
                attackStrat2.setName(attackStrat.getName());
                attackStrat2.setRandomGenerator(attackStrat.getRandomGenerator());
                attackList.add(attackStrat2);
            }
            strategy.setAttackStrategyVector(attackList);
            
            it = strat.getAskingStrategyVector().iterator();
            Vector askingList = new Vector();
            while (it.hasNext()) {
                AskingStrategy askingStrat = (AskingStrategy)it.next();
                AskingStrategy askingStrat2 = (AskingStrategy)askingStrat.getClass().newInstance();
                askingStrat2.setMemory(askingStrat.getMemory());
                askingStrat2.setName(askingStrat.getName());
                askingStrat2.setRandomGenerator(askingStrat.getRandomGenerator());
                askingList.add(askingStrat2);
            }
            strategy.setAskingStrategyVector(askingList);
            
            it = strat.getDecisionStrategyVector().iterator();
            Vector decisionList = new Vector();
            while (it.hasNext()) {
                DecisionSupportStrategy decisionStrat = (DecisionSupportStrategy)it.next();
                DecisionSupportStrategy decisionStrat2 = (DecisionSupportStrategy)decisionStrat.getClass().newInstance();
                decisionStrat2.setMemory(decisionStrat.getMemory());
                decisionStrat2.setName(decisionStrat.getName());
                decisionStrat2.setRandomGenerator(decisionStrat.getRandomGenerator());
                decisionList.add(decisionStrat2);
            }
            strategy.setDecisionStrategyVector(decisionList);
            
            //strategy.setAskingStrategyVector((Vector)strat.getAskingStrategyVector().clone());
            //strategy.setAttackStrategyVector((Vector)strat.getAttackStrategyVector().clone());
            //strategy.setDecisionStrategyVector((Vector)strat.getDecisionStrategyVector().clone());
        }catch(InstantiationException e) {
            System.out.println(e.toString());
        }catch(IllegalAccessException e) {
            System.out.println(e.toString());
            //}catch(NoSuchMethodException e) {
            //  System.out.println(e.toString());
            //}catch(java.lang.reflect.InvocationTargetException e) {
            //  System.out.println(e.toString());
        }
        for(int i=0;i<players.length;i++) {
            Player player = (Player)players[i];
            // check if the name isn't already in use
            if (!(player.getName().equals(playerNameTextField.getText()))) {
                if (checkName(playerList,playerNameTextField.getText())) {
                    player.setName(playerNameTextField.getText());
                } else {
                    MsgBox msgBox = new MsgBox(null, true, "Unable to change name of "+player.getName(), "The name "+playerNameTextField.getText()+" is already in use",false);
                    msgBox.setVisible(true);
                    continue;
                }
            }
            int index = playerList.indexOf(player);
            playerList.remove(player);
            player.setStrategy(strategy);
            Double hp = null;
            try {
                hp = (Double)playerPointsSpinner.getValue();
            }catch(NumberFormatException e) {
                System.out.println(e.toString());
                return;
            }
            player.setHitPoints(hp.floatValue());
            playerList.add(index,player);
        }
        refreshListSummary();
        playerJList.setListData(playerList.toArray());
        playerNameTextField.setText("");
    }//GEN-LAST:event_affectButtonActionPerformed
    
    /** refresh the list conataing the strategies-players associations **/
    private void refreshListSummary() {
        ArrayList l = new ArrayList();
        Iterator it = playerList.iterator();
        while (it.hasNext()) {
            Player player =  (Player)it.next();
            l.add(player.toString()+" : "+player.getStrategy().toString());
        }
        affectedStrategyJList.setListData(l.toArray());
    }
    
    private void beginPointsTextFieldKeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_beginPointsTextFieldKeyTyped
        try {
            Integer integer = new Integer(beginPointsTextField.getText());
        }catch(NumberFormatException e) {
            return;
        }
    }//GEN-LAST:event_beginPointsTextFieldKeyTyped
    
    private void finishButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_finishButtonActionPerformed
        returnStatus=1;
        closeDialog(null);
    }//GEN-LAST:event_finishButtonActionPerformed
    
    private void nextButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nextButtonActionPerformed
        Integer playersNumber = null;
        try {
            playersNumber = (Integer)playersNumberSpinner.getValue();
        }catch(NumberFormatException e) {
            System.out.println(e.toString());
            return;
        }
        Float beginPoints = null;
        try {
            beginPoints = new Float(beginPointsTextField.getText());
        }catch(NumberFormatException e) {
            System.out.println(e.toString());
            return;
        }
        playerPointsSpinner.setValue(beginPoints);
        Integer trace = null;
        try {
            trace = (Integer)traceLevelSpinner.getValue();
        }catch(NumberFormatException e) {
            System.out.println(e.toString());
            return;
        }
        traceLevel=trace.intValue();
        if (simulations.SimulationParameters.debug) {
            System.out.println("trace level = "+traceLevel);
        }
        Integer randomSeed = null;
        if (timeCheckBox.isSelected()) {
            seed = new GregorianCalendar().get(Calendar.MILLISECOND);
        } else {
            try {
                randomSeed = (Integer)randomSeedSpinner.getValue();
            }catch(NumberFormatException e) {
                System.out.println(e.toString());
                return;
            }
            seed=randomSeed.intValue();
        }
        Float coalitionalPts = null;
        try {
            coalitionalPts = new Float(coalitionPointsTextField.getText());
        }catch(NumberFormatException e) {
            System.out.println(e.toString());
            return;
        }
        coalitionPoints=coalitionalPts.floatValue();
        
        Double winnerPts = null;
        try {
            winnerPts = (Double)winnerPointsSpinner.getValue();
        }catch(NumberFormatException e) {
            System.out.println(e.toString());
            return;
        }
        winnerPoints=winnerPts.floatValue();
        
        Integer loosingPts = null;
        try {
            loosingPts = new Integer(loosingPointsTextField.getText());
        }catch(NumberFormatException e) {
            System.out.println(e.toString());
            return;
        }
        loosingPoints=loosingPts.intValue();
        
        playerList = new ArrayList();
        for(int i=0;i<playersNumber.intValue();i++) {
            playerList.add(new  Player("Player #"+i, beginPoints.floatValue(), new RandomStrategy()));
        }
        tabbedPane.setEnabledAt(step-1,false);
        tabbedPane.setEnabledAt(step, true);
        step++;
        tabbedPane.setSelectedIndex(step-1);
        previousButton.setEnabled(true);
        nextButton.setEnabled(false);
        finishButton.setEnabled(true);
        saveButton.setEnabled(true);
        if (activatedCheckBox.isSelected()) {
            proportional=true;
        } else {
            proportional=false;
        }
        // loading players list
        playerJList.setListData(playerList.toArray());
        refreshListSummary();
        if (coalitionDef1RadioButton.isSelected()) {
            coalitionDefinition=0;
        } else if (coalitionDef2RadioButton.isSelected()) {
            coalitionDefinition=1;
        } else if (coalitionDef3RadioButton.isSelected()) {
            coalitionDefinition=2;
        } else if (coalitionDef4RadioButton.isSelected()) {
            coalitionDefinition=3;
        } else {
            coalitionDefinition=4;
        }
    }//GEN-LAST:event_nextButtonActionPerformed
    
    private void previousButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_previousButtonActionPerformed
        step--;
        tabbedPane.setEnabledAt(step-1, true);
        tabbedPane.setEnabledAt(step, false);
        tabbedPane.setSelectedIndex(step-1);
        previousButton.setEnabled(false);
        nextButton.setEnabled(true);
        finishButton.setEnabled(false);
        saveButton.setEnabled(false);
    }//GEN-LAST:event_previousButtonActionPerformed
    
    private void timeCheckBoxMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_timeCheckBoxMouseClicked
        // Add your handling code here:
        if (timeCheckBox.isSelected()) {
            randomSeedSpinner.setEnabled(false);
        } else {
            randomSeedSpinner.setEnabled(true);
        }
    }//GEN-LAST:event_timeCheckBoxMouseClicked
    
    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
        // Add your handling code here:
        closeDialog(null);
    }//GEN-LAST:event_cancelButtonActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog
    
    /** Getter for property seed.
     * @return Value of property seed.
     *
     */
    public long getSeed() {
        return seed;
    }
    
    /** Setter for property seed.
     * @param seed New value of property seed.
     *
     */
    public void setSeed(long seed) {
        this.seed = seed;
    }
    
    /** Getter for property playerList.
     * @return Value of property playerList.
     *
     */
    public java.util.ArrayList getPlayerList() {
        return playerList;
    }
    
    /** Setter for property playerList.
     * @param playerList New value of property playerList.
     *
     */
    public void setPlayerList(java.util.ArrayList playerList) {
        this.playerList = playerList;
    }
    
    /** Getter for property returnStatus.
     * @return Value of property returnStatus.
     *
     */
    public int getReturnStatus() {
        return returnStatus;
    }
    
    /** Setter for property returnStatus.
     * @param returnStatus New value of property returnStatus.
     *
     */
    public void setReturnStatus(int returnStatus) {
        this.returnStatus = returnStatus;
    }
    
    /** Getter for property coalitionalPoints.
     * @return Value of property coalitionalPoints.
     *
     */
    public float getCoalitionPoints() {
        return coalitionPoints;
    }
    
    /** Setter for property coalitionalPoints.
     * @param coalitionPoints The coalitions points
     */
    public void setCoalitionPoints(float coalitionPoints) {
        this.coalitionPoints = coalitionPoints;
    }
    
    /** Getter for property loosingPoints.
     * @return Value of property loosingPoints.
     *
     */
    public int getLoosingPoints() {
        return loosingPoints;
    }
    
    /** Setter for property loosingPoints.
     * @param loosingPoints New value of property loosingPoints.
     *
     */
    public void setLoosingPoints(int loosingPoints) {
        this.loosingPoints = loosingPoints;
    }
    
    /** Getter for property proportional.
     * @return Value of property proportional.
     *
     */
    public boolean isProportional() {
        return proportional;
    }
    
    /** Setter for property proportional.
     * @param proportional New value of property proportional.
     *
     */
    public void setProportional(boolean proportional) {
        this.proportional = proportional;
    }
    
    /** Getter for property winnerPoints.
     * @return Value of property winnerPoints.
     *
     */
    public float getWinnerPoints() {
        return winnerPoints;
    }
    
    /** Setter for property winnerPoints.
     * @param winnerPoints New value of property winnerPoints.
     *
     */
    public void setWinnerPoints(float winnerPoints) {
        this.winnerPoints = winnerPoints;
    }
    
    /** Getter for property traceLevel.
     * @return Value of property traceLevel.
     *
     */
    public int getTraceLevel() {
        return traceLevel;
    }
    
    /** Setter for property traceLevel.
     * @param traceLevel New value of property traceLevel.
     *
     */
    public void setTraceLevel(int traceLevel) {
        this.traceLevel = traceLevel;
    }
    
    /** Getter for property historyFile.
     * @return Value of property historyFile.
     *
     */
    public java.io.File getHistoryFile() {
        return historyFile;
    }
    
    /** Setter for property historyFile.
     * @param historyFile New value of property historyFile.
     *
     */
    public void setHistoryFile(java.io.File historyFile) {
        this.historyFile = historyFile;
    }
    
    /**
     * Getter for property coalitionDefinition.
     * @return Value of property coalitionDefinition.
     */
    public int getCoalitionDefinition() {
        return coalitionDefinition;
    }
    
    /**
     * Setter for property coalitionDefinition.
     * @param coalitionDefinition New value of property coalitionDefinition.
     */
    public void setCoalitionDefinition(int coalitionDefinition) {
        this.coalitionDefinition = coalitionDefinition;
    }
    
    /**
     * Getter for property graphFile.
     * @return Value of property graphFile.
     */
    public java.io.File getGraphFile() {
        return graphFile;
    }
    
    /**
     * Setter for property graphFile.
     * @param graphFile New value of property graphFile.
     */
    public void setGraphFile(java.io.File graphFile) {
        this.graphFile = graphFile;
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton LoadFromFileButton;
    private javax.swing.JLabel TitleLabel;
    private javax.swing.JCheckBox activatedCheckBox;
    private javax.swing.JButton affectButton;
    private javax.swing.JList affectedStrategyJList;
    private javax.swing.JLabel affectedStrategyLabel;
    private javax.swing.JScrollPane affectedStrategyScrollPane;
    private javax.swing.JLabel beginPointsLabel;
    private javax.swing.JTextField beginPointsTextField;
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.JPanel buttonsPanel;
    private javax.swing.JButton cancelButton;
    private javax.swing.JRadioButton coalitionDef1RadioButton;
    private javax.swing.JRadioButton coalitionDef2RadioButton;
    private javax.swing.JRadioButton coalitionDef3RadioButton;
    private javax.swing.JRadioButton coalitionDef4RadioButton;
    private javax.swing.JRadioButton coalitionDef5RadioButton;
    private javax.swing.JTextField coalitionPointsTextField;
    private javax.swing.JTextArea descriptionTextArea;
    private javax.swing.JButton editButton;
    private javax.swing.JButton finishButton;
    private javax.swing.JPanel firstPanel;
    private javax.swing.JButton graphPathButton;
    private javax.swing.JTextField graphPathTextField;
    private javax.swing.JButton historyPathButton;
    private javax.swing.JTextField historyPathTextField;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JLabel labelPlayerNumber;
    private javax.swing.JLabel labelPlayerNumber1;
    private javax.swing.JLabel labelPlayerNumber2;
    private javax.swing.JButton loadFromFolderButton;
    private javax.swing.JTextField loosingPointsTextField;
    private javax.swing.JButton nextButton;
    private javax.swing.JList playerJList;
    private javax.swing.JTextField playerNameTextField;
    private javax.swing.JSpinner playerPointsSpinner;
    private javax.swing.JScrollPane playerScrollPane;
    private javax.swing.JSpinner playersNumberSpinner;
    private javax.swing.JButton previousButton;
    private javax.swing.JLabel randomSeedLabel;
    private javax.swing.JSpinner randomSeedSpinner;
    private javax.swing.JButton saveButton;
    private javax.swing.JLabel selectPlayerLabel;
    private javax.swing.JLabel selectStrategyLabel;
    private javax.swing.JList strategyJList;
    private javax.swing.JPanel strategyPanel;
    private javax.swing.JScrollPane strategyScrollPane;
    private javax.swing.JTabbedPane tabbedPane;
    private javax.swing.JCheckBox timeCheckBox;
    private javax.swing.JSpinner traceLevelSpinner;
    private javax.swing.JSpinner winnerPointsSpinner;
    // End of variables declaration//GEN-END:variables
    
}
