/*
 * CoalitionCanvas2.java
 *
 * Created on April 16, 2004, 2:58 AM
 */

package simulations.gui.coalitions;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics;

/** This method describes a second coalition canvas to show quickly who attack who
 * directly or indirectly
 * @author tom
 */
public class CoalitionCanvas2 extends Canvas {
    
    private boolean[][] table;
    
    /** Creates a new instance of CoalitionCanvas2
     * @param size The number of players
     */
    public CoalitionCanvas2(int size) {
        this.setBackground(new java.awt.Color(255, 255, 255));
        table = new boolean[size][size];
    }
    
    /** Overload of paint method
     * @param g The graphics
     */    
    public void paint(Graphics g) {
        drawGrid(g);
        int y=1;
        Color black = Color.BLACK;
        for(int i=0;i<table[0].length;i++) {
            int x=1;
            for(int j=0;j<table[0].length;j++) {
                if (table[i][j]) {
                    drawCube(g, x, y, black);
                }
                x+=4;
            }
            y+=4;
        }
    }
    
    /** This method draw a cube at point (x,y) filled with color
     * @param g The graphics
     * @param x The x point
     * @param y The y point
     * @param color The color to use
     */    
    public void drawCube(Graphics g, int x, int y, Color color) {
        g.setColor(color);
        g.drawLine(x, y, x+2, y);
        g.drawLine(x,y+1, x+2, y+1);
        g.drawLine(x,y+2, x+2, y+2);
    }
    
    /** Draw the grid
     * @param g The graphics
     */
    public void drawGrid(Graphics g) {
        int dim=table[0].length;
        int size = 4*dim;
        for(int i=0;i<dim;i++) {
            g.drawLine(0, 4*i, size, 4*i);
            g.drawLine(4*i, 0, 4*i,size);
        }
        g.drawLine(0, size, size, size);
        g.drawLine(size, 0, size, size);
    }
    
    /** Getter for property table.
     * @return Value of property table.
     *
     */
    public boolean[][] getTable() {
        return this.table;
    }    
    
    /** Setter for property table.
     * @param table New value of property table.
     *
     */
    public void setTable(boolean[][] table) {
        this.table = table;
    }    
    
}
