/*
 * CoalitionHistoryDialog.java
 *
 * Created on May 26, 2004, 11:55 PM
 */

package simulations.gui.coalitions;

import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.SpinnerNumberModel;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import java.awt.GridBagConstraints;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import simulations.gui.StrategyFileFilter;

/**
 * This class describes a dialog that summary the coalition formation history in the game
 * @author  tom
 */
public class CoalitionHistoryDialog extends javax.swing.JDialog {
    
    /** The list of all coalitions **/
    private ArrayList coalitionList;
    
    /** The turn to display coalitions **/
    private int turn;
    
    /** The coalition definition **/
    private String coalitionDef;
    
    /** Creates new form CoalitionHistoryDialog
     * @param parent The parent frame
     * @param modal True is the dialog is modal
     * @param coalitionList The list of coalitions in the game
     **/
    public CoalitionHistoryDialog(java.awt.Frame parent, boolean modal, ArrayList coalitionList, String coalitionDef) {
        super(parent, modal);
        initComponents();
        this.coalitionList=coalitionList;
        this.coalitionDef=coalitionDef;
        if (simulations.SimulationParameters.debug) {
            System.out.println("coalitionList.size() = "+coalitionList.size());
        }
        turn=0;
        display();
        turnSpinner.setModel(new SpinnerNumberModel(0,0,coalitionList.size(),1));
        this.setSize(700,480);
        this.setLocationRelativeTo(parent);
    }
    
    /** Display the coalitions at turn **/
    private void display() {
        if (simulations.SimulationParameters.debug) {
            System.out.println("display turn = "+turn);
        }
        coalitionPanel.removeAll();
        ArrayList l = (ArrayList)coalitionList.get(turn);
        if (simulations.SimulationParameters.debug) {
            System.out.println("coalition.size() = "+l.size());
        }
        Iterator it = l.iterator();
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx=0;
        gbc.gridy=0;
        gbc.anchor=GridBagConstraints.WEST;
        gbc.fill=GridBagConstraints.HORIZONTAL;
        int y = 0;
        while (it.hasNext()) {
            ArrayList l2 = (ArrayList)it.next();
            JLabel label1 = new JLabel("Coalition #"+(y+1));
            gbc.gridx=0;
            gbc.gridy=y;
            coalitionPanel.add(label1,gbc);
            JLabel label2 = new JLabel(l2.toString());
            gbc.gridx=1;
            coalitionPanel.add(label2,gbc);
            y++;
        }
        coalitionPanel.repaint();
        //centerPanel.repaint();
        //this.repaint();
        this.validate();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jLabel1 = new javax.swing.JLabel();
        jPanel1 = new javax.swing.JPanel();
        saveButton = new javax.swing.JButton();
        closeButton = new javax.swing.JButton();
        centerPanel = new javax.swing.JPanel();
        jPanel5 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        turnSpinner = new javax.swing.JSpinner();
        goButton = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        coalitionPanel = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        nextButton = new javax.swing.JButton();
        jPanel4 = new javax.swing.JPanel();
        previousButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Coalition history");
        jLabel1.setFont(new java.awt.Font("Dialog", 1, 18));
        jLabel1.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/edit.png")));
        jLabel1.setText("Coalitions history");
        getContentPane().add(jLabel1, java.awt.BorderLayout.NORTH);

        saveButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/save.png")));
        saveButton.setText("Save");
        saveButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveButtonActionPerformed(evt);
            }
        });

        jPanel1.add(saveButton);

        closeButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/button_cancel.png")));
        closeButton.setText("Close");
        closeButton.setToolTipText("Close the dialog");
        closeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeButtonActionPerformed(evt);
            }
        });

        jPanel1.add(closeButton);

        getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

        centerPanel.setLayout(new java.awt.BorderLayout());

        jLabel2.setText("Coalitions at turn # ");
        jPanel5.add(jLabel2);

        jPanel5.add(turnSpinner);

        goButton.setText("Go");
        goButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                goButtonActionPerformed(evt);
            }
        });

        jPanel5.add(goButton);

        centerPanel.add(jPanel5, java.awt.BorderLayout.SOUTH);

        coalitionPanel.setLayout(new java.awt.GridBagLayout());

        jScrollPane1.setViewportView(coalitionPanel);

        centerPanel.add(jScrollPane1, java.awt.BorderLayout.CENTER);

        getContentPane().add(centerPanel, java.awt.BorderLayout.CENTER);

        jPanel3.setLayout(new java.awt.GridBagLayout());

        nextButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/forward.png")));
        nextButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nextButtonActionPerformed(evt);
            }
        });

        jPanel3.add(nextButton, new java.awt.GridBagConstraints());

        getContentPane().add(jPanel3, java.awt.BorderLayout.EAST);

        jPanel4.setLayout(new java.awt.GridBagLayout());

        previousButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/back.png")));
        previousButton.setEnabled(false);
        previousButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                previousButtonActionPerformed(evt);
            }
        });

        jPanel4.add(previousButton, new java.awt.GridBagConstraints());

        getContentPane().add(jPanel4, java.awt.BorderLayout.WEST);

        pack();
    }//GEN-END:initComponents

    private void saveButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveButtonActionPerformed
        // TODO add your handling code here:
        JFileChooser chooser = new JFileChooser();
        StrategyFileFilter filter = new StrategyFileFilter("html");
        filter.setDescription("HTML files");
        chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            if (simulations.SimulationParameters.debug) {
                System.out.println("You chose to save this file: " +
                chooser.getSelectedFile().getName());
            }
            String fileName = chooser.getSelectedFile().getPath();
            if (simulations.SimulationParameters.debug) {
                System.out.println(fileName);
            }
            if (!(fileName.substring(fileName.length()-5,fileName.length()).equals(".html"))) {
                fileName=fileName+".html";
            }
            save(new File(fileName));
        }
    }//GEN-LAST:event_saveButtonActionPerformed

    public void save(File f) {
        try {
            if (simulations.SimulationParameters.debug) {
                System.out.println("saving coalition history in "+f.getPath());
            }
            FileWriter writer = new FileWriter(f);
            writer.write("<html>\n");
            writer.write("<title>Coalition history</title>\n");
            writer.write("<body>\n");
            writer.write("<center><h2>Coalition history</h2></center>\n");
            writer.write("<br>\n");
            writer.write("<u><h3>Coalition definition : </h3></u>");
            writer.write(coalitionDef);
            writer.write("<br><br>");
            writer.write("<center><table border>\n");
            writer.write("<tr><td>Turn</td><td>Coalition #</td><td>Coalition</td></tr>\n");
            Iterator it = coalitionList.iterator();
            int cptTurn=1;
            while (it.hasNext()) {
                ArrayList coalList = (ArrayList)it.next();
                Iterator it2 = coalList.iterator();
                int cptCoal=1;
                while (it2.hasNext()) {
                    ArrayList l = (ArrayList)it2.next();
                    if (cptCoal==1) {
                        writer.write("<tr><td>Turn #"+cptTurn+"</td><td>Coalition #1</td><td>"+l.toString()+"</td></tr>");
                    } else {
                        writer.write("<tr><td></td><td>Coalition #"+cptCoal+"</td><td>"+l.toString()+"</td></tr>");
                    }
                    cptCoal++;
                }
                cptTurn++;
            }
            writer.write("</table></center>\n");
            writer.write("</body>\n");
            writer.write("</html>\n");
            writer.close();
        }catch(IOException e) {
            System.out.println(e.toString());
            System.out.println("unable to write file");
        }
    }
    
    private void goButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_goButtonActionPerformed
        // TODO add your handling code here:
        turn = ((Integer)turnSpinner.getValue()).intValue();
        display();
    }//GEN-LAST:event_goButtonActionPerformed

    private void previousButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_previousButtonActionPerformed
        // TODO add your handling code here:
        if (turn<=1) {
            previousButton.setEnabled(false);
            turn--;
        } else {
            nextButton.setEnabled(true);
            turn--;
        }
        turnSpinner.setValue(new Integer(turn));
        display();
    }//GEN-LAST:event_previousButtonActionPerformed

    private void nextButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nextButtonActionPerformed
        // TODO add your handling code here:
        if (turn>=(coalitionList.size()-2)) {
            nextButton.setEnabled(false);
            turn++;
        } else {
            previousButton.setEnabled(true);
            turn++;
        }
        turnSpinner.setValue(new Integer(turn));
        display();
    }//GEN-LAST:event_nextButtonActionPerformed
    
    private void closeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeButtonActionPerformed
        // TODO add your handling code here:
        this.setVisible(false);
        this.dispose();
    }//GEN-LAST:event_closeButtonActionPerformed
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel centerPanel;
    private javax.swing.JButton closeButton;
    private javax.swing.JPanel coalitionPanel;
    private javax.swing.JButton goButton;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JButton nextButton;
    private javax.swing.JButton previousButton;
    private javax.swing.JButton saveButton;
    private javax.swing.JSpinner turnSpinner;
    // End of variables declaration//GEN-END:variables
    
}
