/*
 * CoalitionRankingSaver.java
 *
 * Created on 31 mai 2004, 13:58
 */

package simulations.gui.coalitions;

import java.util.ArrayList;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map.Entry;

import simulations.core.strategy.ObjectObjectElement;
import simulations.gui.StrategyFileFilter;

import javax.swing.JFileChooser;

import javax.swing.JLabel;
import javax.swing.JTable;

import java.io.File;
import java.io.IOException;
import java.io.FileWriter;


/**
 *
 * @author  tom
 */
public class CoalitionRankingSaver {
    
    /** The coalition list list **/
    private Hashtable coalitionRanking;
    
    /** All coalitions **/
    private ArrayList allCoalitions;
    
    /** The ordered coalition list **/
    private ArrayList orderedCoalitionList;
    
    /** The table containing ranking **/
    private JTable table;
    
    /** The vector cantaining the data **/
    private Vector dataVector;
    
    /** Creates a new instance of CoalitionRankingSaver */
    public CoalitionRankingSaver(Hashtable coalitionRanking, String def) {
        this.coalitionRanking=coalitionRanking;
        initAllCoalitions();
        orderCoalitionList();
        display();
    }
    
    private void display() {
        Vector titleVector = new Vector();
        titleVector.add(new String("rank"));
        titleVector.add(new String("coalition"));
        titleVector.add(new String("nb occ"));
        titleVector.add(new String("nb seq occ"));
        titleVector.add(new String("lg max"));
        dataVector = new Vector();
        Iterator it = orderedCoalitionList.iterator();
        int cpt = 1;
        while (it.hasNext()) {
            ObjectObjectElement elt = (ObjectObjectElement)it.next();
            CoalitionData data = (CoalitionData)elt.getObject2();
            Vector line = new Vector();
            line.add(new String(""+cpt));
            line.add(new String(elt.getObject1().toString()));
            line.add(new String(""+data.getNbOcc()));
            line.add(new String(""+data.getSeqNbOcc()));
            line.add(new String(""+data.getMaxLength()));
            dataVector.add(line);
            cpt++;
        }
    }
    
    /** Init the list of all coalitions **/
    private void initAllCoalitions() {
        allCoalitions = new ArrayList();
        Iterator it = coalitionRanking.entrySet().iterator();
        while (it.hasNext()) {
            Entry entry = (Entry)it.next();
            allCoalitions.add(((ArrayList)entry.getKey()).clone());
        }
    }
    
    private void orderCoalitionList() {
        orderedCoalitionList = new ArrayList();
        for(int i=0;i<allCoalitions.size();i++) {
            try {
                int max=-1;
                int maxIndex = -1;
                for(int j=0;j<allCoalitions.size();j++) {
                    CoalitionData data = (CoalitionData)coalitionRanking.get((ArrayList)allCoalitions.get(j));
                    if (data.getNbOcc()>max) {
                        maxIndex=j;
                        max=data.getNbOcc();
                    }
                }
                orderedCoalitionList.add(
                new ObjectObjectElement(allCoalitions.get(maxIndex),coalitionRanking.get(allCoalitions.get(maxIndex))));
                allCoalitions.remove(maxIndex);
                i--;
            }catch(ArrayIndexOutOfBoundsException e) {
                continue;
            }
        }
    }
    
    /** Save the table in html file f
     * @param f The file to save the table
     **/
    public void save(File f) {
        try {
            FileWriter writer = new FileWriter(f);
            writer.write("<html>\n");
            writer.write("<title>Coalition ranking</title>\n");
            writer.write("<body>\n");
            writer.write("<h2><center>Coalition ranking</center></h2>\n");
            writer.write("<br>\n");
            writer.write("<center><table border>\n");
            writer.write("<tr><td>Rank</td><td>Coalition</td><td>Nb occ</td><td>Nb seq occ</td><td>Lg max</td></tr>\n");
            Iterator it = dataVector.iterator();
            while (it.hasNext()) {
                Vector line = (Vector)it.next();
                writer.write("<tr><td>"+line.get(0).toString()+
                "</td><td>"+line.get(1).toString()+
                "</td><td>"+line.get(2).toString()+
                "</td><td>"+line.get(3).toString()+
                "</td><td>"+line.get(4).toString()+
                "</td></tr>\n");
            }
            writer.write("</table></center>\n");
            writer.write("</body>\n");
            writer.write("</html>\n");
            writer.close();
        }catch(IOException e) {
            
        }
    }
    
}
