/*
 * CoalitionTable.java
 *
 * Created on April 16, 2004, 2:21 AM
 */

package simulations.gui.coalitions;

import java.util.Iterator;
import java.util.ArrayList;

import simulations.core.history.Turn;
import simulations.core.history.PlayerTurn;
import simulations.core.Player;

/** This class is used to store who attack who
 * directly or indirectly
 * @author tom
 */
public class CoalitionTable {
    
    private boolean[][] table;
    
    private ArrayList playerList;
    
    /** Creates a new instance of CoalitionTable
     * @param playerList The players list
     */
    public CoalitionTable(ArrayList playerList) {
        table = new boolean[playerList.size()][playerList.size()];
        this.playerList = playerList;
    }
    
    /** Returns the list of players that have attacked player
     * @param player The player
     * @return Returns the list of players that have attacked player
     */    
    public ArrayList getAttackingPlayers(Player player) {
        int index = playerList.indexOf(player);
        ArrayList l = new ArrayList();
        for(int i=0;i<playerList.size();i++) {
            if (table[i][index]) {
                l.add(playerList.get(i));
            }
        }
        return l;
    }
    
    /** This method init the table with the turn
     * @param turn The turn to use
     */    
    public void setTable(Turn turn) {
        for(int i=0;i<(playerList.size()-1);i++) {
            for(int j=0;j<(playerList.size()-1);j++) {
                table[i][j]=false;
            }
        }
        Iterator it = turn.getPlayersTurnsList().iterator();
        while (it.hasNext()) {
            PlayerTurn pt = (PlayerTurn)it.next();
            int indexAttackingPlayer = playerList.indexOf(pt.getPlayer());
            int indexAttackedPlayer = playerList.indexOf(pt.getAttackedPlayer());
            table[indexAttackingPlayer][indexAttackedPlayer]=true;
        }
    }
    
    /** Getter for property playerList.
     * @return Value of property playerList.
     *
     */
    public java.util.ArrayList getPlayerList() {
        return playerList;
    }    
    
    /** Setter for property playerList.
     * @param playerList New value of property playerList.
     *
     */
    public void setPlayerList(java.util.ArrayList playerList) {
        this.playerList = playerList;
    }
    
    /** Getter for property table.
     * @return Value of property table.
     *
     */
    public boolean[][] getTable() {
        return this.table;
    }
    
    /** Setter for property table.
     * @param table New value of property table.
     *
     */
    public void setTable(boolean[][] table) {
        this.table = table;
    }
    
}
