/*
 * CoalitionView.java
 *
 * Created on March 29, 2004, 5:20 PM
 */

package simulations.gui.coalitions;

import simulations.gui.curves.ColorsDialog;
import simulations.core.history.Turn;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Vector;
import java.awt.Graphics;

import javax.swing.JLabel;

/** This class describes the dialog box to visualize the formation coalition
 * @author tom
 */
public class CoalitionView extends javax.swing.JDialog implements Runnable {
    
    /** The canvas containing the graph **/
    private CoalitionCanvas coalitionCanvas;
    
    /** The canvas containing the table **/
    private CoalitionCanvas2 coalitionCanvas2;
    
    /** The coalition table **/
    private CoalitionTable coalitionTable;
    
    /** Creates new form CoalitionView
     * @param playerList The players list
     * @param parent The parent frame
     * @param modal true if the dialog is modal
     * @param size The size of the grid
     */
    public CoalitionView(java.awt.Frame parent, boolean modal, ArrayList playerList, int size) {
        super(parent, modal);
        initComponents();
        coalitionCanvas = new CoalitionCanvas(playerList, size);
        scrollPane1.setViewportView(coalitionCanvas);
        coalitionCanvas2 = new CoalitionCanvas2(playerList.size());
        coalitionTable = new CoalitionTable(playerList);
        scrollPane2.setViewportView(coalitionCanvas2);
        this.setSize(800, 600);
    }
    
    /** display the coalition list graphicly
     * @param turn The turn used to display infos
     * @param turnDescription The turn description to display
     * @param coalitionList The coalition list to display
     */
    public void display(ArrayList coalitionList, Turn turn, String turnDescription) {
        coalitionCanvas.setCoalitionList(coalitionList);
        coalitionCanvas.repaint();
        coalitionTable.setTable(turn);
        coalitionCanvas2.setTable(coalitionTable.getTable());
        coalitionCanvas2.repaint();
        textAreaDescription.setText(turnDescription);
        coalitionsPanel.removeAll();
        coalitionsPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gbc = new java.awt.GridBagConstraints();
        gbc.gridx=0;
        gbc.gridy=0;
        gbc.anchor=gbc.WEST;
        gbc.fill=gbc.HORIZONTAL;
        Iterator it = coalitionList.iterator();
        int cpt=1;
        while (it.hasNext()) {
            ArrayList l = (ArrayList)it.next();
            gbc.gridx=0;
            gbc.gridy=(cpt-1);
            JLabel label1 = new JLabel("Coalition #"+cpt);
            coalitionsPanel.add(label1,gbc);
            JLabel label2 = new JLabel(l.toString());
            gbc.gridx=1;
            coalitionsPanel.add(label2,gbc);
            cpt++;
        }
        coalitionsPanel.repaint();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jPanel1 = new javax.swing.JPanel();
        closeButton = new javax.swing.JButton();
        colorsButton = new javax.swing.JButton();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        scrollPane1 = new javax.swing.JScrollPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        coalitionsPanel = new javax.swing.JPanel();
        scrollPane2 = new javax.swing.JScrollPane();
        scrollPane3 = new javax.swing.JScrollPane();
        textAreaDescription = new javax.swing.JTextArea();

        setTitle("Coalitions");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        jPanel1.setBackground(new java.awt.Color(255, 255, 255));
        closeButton.setBackground(new java.awt.Color(255, 255, 255));
        closeButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/button_cancel.png")));
        closeButton.setText("Close");
        closeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeButtonActionPerformed(evt);
            }
        });

        jPanel1.add(closeButton);

        colorsButton.setBackground(new java.awt.Color(255, 255, 255));
        colorsButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/colors.png")));
        colorsButton.setText("Colors");
        colorsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                colorsButtonActionPerformed(evt);
            }
        });

        jPanel1.add(colorsButton);

        getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

        jTabbedPane1.setBackground(new java.awt.Color(255, 255, 255));
        jTabbedPane1.setTabPlacement(javax.swing.JTabbedPane.BOTTOM);
        scrollPane1.setBackground(new java.awt.Color(255, 255, 255));
        jTabbedPane1.addTab("Coalitions", scrollPane1);

        coalitionsPanel.setLayout(new java.awt.GridLayout());

        jScrollPane1.setViewportView(coalitionsPanel);

        jTabbedPane1.addTab("Coalitions", jScrollPane1);

        scrollPane2.setBackground(new java.awt.Color(255, 255, 255));
        jTabbedPane1.addTab("Players attacks", scrollPane2);

        scrollPane3.setBackground(new java.awt.Color(255, 255, 255));
        scrollPane3.setViewportView(textAreaDescription);

        jTabbedPane1.addTab("Turn description", scrollPane3);

        getContentPane().add(jTabbedPane1, java.awt.BorderLayout.CENTER);

        pack();
    }//GEN-END:initComponents

    private void colorsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_colorsButtonActionPerformed
        // Add your handling code here:
        //new ColorsDialog(null, false, coalitionCanvas.getColorsPlayerVector()).show();
    }//GEN-LAST:event_colorsButtonActionPerformed

    /** This method is called on a click on the close button
     * @param evt The event
     **/
    private void closeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeButtonActionPerformed
        // Add your handling code here:
        closeDialog(null);
    }//GEN-LAST:event_closeButtonActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog

    /** implementation of the run method defined by the interface Runnable **/
    public void run() {
        this.setVisible(true);
    }    
        
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton closeButton;
    private javax.swing.JPanel coalitionsPanel;
    private javax.swing.JButton colorsButton;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JScrollPane scrollPane1;
    private javax.swing.JScrollPane scrollPane2;
    private javax.swing.JScrollPane scrollPane3;
    private javax.swing.JTextArea textAreaDescription;
    // End of variables declaration//GEN-END:variables
    
}
