/*
 * CoalitionsRankingDialog.java
 *
 * Created on May 20, 2004, 3:19 AM
 */

package simulations.gui.coalitions;

import java.util.ArrayList;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map.Entry;

import simulations.core.strategy.ObjectObjectElement;
import simulations.gui.StrategyFileFilter;

import javax.swing.JFileChooser;

import javax.swing.JLabel;
import javax.swing.JTable;

import java.io.File;
import java.io.IOException;
import java.io.FileWriter;

/**
 * This class describes a dialog to display the ranking of coalitions
 * @author  tom
 */
public class CoalitionsRankingDialog extends javax.swing.JDialog {
    
    /** The coalition list list **/
    private Hashtable coalitionRanking;
    
    /** All coalitions **/
    private ArrayList allCoalitions;
    
    /** The ordered coalition list **/
    private ArrayList orderedCoalitionList;
    
    /** The table containing ranking **/
    private JTable table;
    
    /** The vector cantaining the data **/
    private Vector dataVector;
    
    /**
     * Creates new form CoalitionsRankingDialog
     * @param def The string coalition definition
     * @param parent The parent window
     * @param modal True if the dialog is modal
     * @param coalitionRanking The data containing the coalitions and the number they appears
     */
    public CoalitionsRankingDialog(java.awt.Frame parent, boolean modal, Hashtable coalitionRanking, String def) {
        super(parent, modal);
        initComponents();
        coalitionDefinitionTextArea.setText(def);
        this.coalitionRanking=coalitionRanking;
        initAllCoalitions();
        orderCoalitionList();
        display();
        this.setSize(600,700);
        this.setLocationRelativeTo(parent);
    }
    
    private void display() {
        Vector titleVector = new Vector();
        titleVector.add(new String("rank"));
        titleVector.add(new String("coalition"));
        titleVector.add(new String("nb occ"));
        titleVector.add(new String("nb seq occ"));
        titleVector.add(new String("lg max"));
        dataVector = new Vector();
        Iterator it = orderedCoalitionList.iterator();
        int cpt = 1;
        while (it.hasNext()) {
            ObjectObjectElement elt = (ObjectObjectElement)it.next();
            CoalitionData data = (CoalitionData)elt.getObject2();
            Vector line = new Vector();
            line.add(new String(""+cpt));
            line.add(new String(elt.getObject1().toString()));
            line.add(new String(""+data.getNbOcc()));
            line.add(new String(""+data.getSeqNbOcc()));
            line.add(new String(""+data.getMaxLength()));
            dataVector.add(line);
            cpt++;
        }
        table = new JTable(dataVector,titleVector);
        scrollPane.setViewportView(table);
    }
    
    /** Init the list of all coalitions **/
    private void initAllCoalitions() {
        allCoalitions = new ArrayList();
        Iterator it = coalitionRanking.entrySet().iterator();
        while (it.hasNext()) {
            Entry entry = (Entry)it.next();
            allCoalitions.add(((ArrayList)entry.getKey()).clone());
        }
    }
    
    private void orderCoalitionList() {
        orderedCoalitionList = new ArrayList();
        for(int i=0;i<allCoalitions.size();i++) {
            try {
                int max=-1;
                int maxIndex = -1;
                for(int j=0;j<allCoalitions.size();j++) {
                    CoalitionData data = (CoalitionData)coalitionRanking.get((ArrayList)allCoalitions.get(j));
                    if (data.getNbOcc()>max) {
                        maxIndex=j;
                        max=data.getNbOcc();
                    }
                }
                orderedCoalitionList.add(
                new ObjectObjectElement(allCoalitions.get(maxIndex),coalitionRanking.get(allCoalitions.get(maxIndex))));
                allCoalitions.remove(maxIndex);
                i--;
            }catch(ArrayIndexOutOfBoundsException e) {
                continue;
            }
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jPanel1 = new javax.swing.JPanel();
        saveButton = new javax.swing.JButton();
        closeButton = new javax.swing.JButton();
        scrollPane = new javax.swing.JScrollPane();
        jPanel2 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        coalitionDefinitionTextArea = new javax.swing.JTextArea();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Coalition ranking");
        saveButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/save.png")));
        saveButton.setText("Save");
        saveButton.setToolTipText("Save the ranking");
        saveButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveButtonActionPerformed(evt);
            }
        });

        jPanel1.add(saveButton);

        closeButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/button_cancel.png")));
        closeButton.setText("Close");
        closeButton.setToolTipText("Close the dialog");
        closeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeButtonActionPerformed(evt);
            }
        });

        jPanel1.add(closeButton);

        getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

        getContentPane().add(scrollPane, java.awt.BorderLayout.CENTER);

        jPanel2.setLayout(new java.awt.BorderLayout());

        jLabel1.setFont(new java.awt.Font("Dialog", 1, 16));
        jLabel1.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/kuser.png")));
        jLabel1.setText("Coalitions ranking");
        jPanel2.add(jLabel1, java.awt.BorderLayout.NORTH);

        coalitionDefinitionTextArea.setEditable(false);
        coalitionDefinitionTextArea.setRows(3);
        jScrollPane1.setViewportView(coalitionDefinitionTextArea);

        jPanel2.add(jScrollPane1, java.awt.BorderLayout.CENTER);

        getContentPane().add(jPanel2, java.awt.BorderLayout.NORTH);

        pack();
    }//GEN-END:initComponents

    private void saveButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveButtonActionPerformed
        // TODO add your handling code here:
        JFileChooser chooser = new JFileChooser();
        StrategyFileFilter filter = new StrategyFileFilter("html");
        filter.setDescription("HTML files");
        chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            if (simulations.SimulationParameters.debug) {
                System.out.println("You chose to save this file: " +
                chooser.getSelectedFile().getName());
            }
            String fileName = chooser.getSelectedFile().getPath();
            if (simulations.SimulationParameters.debug) {
                System.out.println(fileName);
            }
            if (!(fileName.substring(fileName.length()-5,fileName.length()).equals(".html"))) {
                fileName=fileName+".html";
            }
            save(new File(fileName));
        }
    }//GEN-LAST:event_saveButtonActionPerformed

    /** Save the table in html file f
     * @param f The file to save the table
     **/
    public void save(File f) {
        try {
            FileWriter writer = new FileWriter(f);
            writer.write("<html>\n");
            writer.write("<title>Coalition ranking</title>\n");
            writer.write("<body>\n");
            writer.write("<h2><center>Coalition ranking</center></h2>\n");
            writer.write("<br>\n");
            writer.write("<center><table border>\n");
            writer.write("<tr><td>Rank</td><td>Coalition</td><td>Nb occ</td><td>Nb seq occ</td><td>Lg max</td></tr>\n");
            Iterator it = dataVector.iterator();
            while (it.hasNext()) {
                Vector line = (Vector)it.next();
                writer.write("<tr><td>"+line.get(0).toString()+
                "</td><td>"+line.get(1).toString()+
                "</td><td>"+line.get(2).toString()+
                "</td><td>"+line.get(3).toString()+
                "</td><td>"+line.get(4).toString()+
                "</td></tr>\n");
            }
            writer.write("</table></center>\n");
            writer.write("</body>\n");
            writer.write("</html>\n");
            writer.close();
        }catch(IOException e) {
            
        }
    }
    
    private void closeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeButtonActionPerformed
        // TODO add your handling code here:
        this.setVisible(false);
        this.dispose();
    }//GEN-LAST:event_closeButtonActionPerformed
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton closeButton;
    private javax.swing.JTextArea coalitionDefinitionTextArea;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JButton saveButton;
    private javax.swing.JScrollPane scrollPane;
    // End of variables declaration//GEN-END:variables
    
}
