/*
 * CurvesCanvas.java
 *
 * Created on April 5, 2004, 4:50 AM
 */

package simulations.gui.curves;

import java.util.ArrayList;
import java.util.Vector;
import java.awt.Graphics;
import java.util.Iterator;
import java.util.Random;
import java.util.GregorianCalendar;
import java.util.Calendar;
import java.awt.Color;
import java.awt.Point;
import java.awt.Image;

import simulations.core.Player;

/** This class describes the canvas which contains the graph with the curves of the
 * game
 * @author tom
 */
public class CurvesCanvas extends java.awt.Canvas {
    
    /** The player list list **/
    private ArrayList list;
    
    /** The selected players **/
    private ArrayList selectedPlayers;
    
    /** This vector store the colors used for each player **/
    private Vector colorsVector;
    
    /** This vector store the last point for each player
     * in order to draw draw a curve and not a suite of points
     **/
    private Vector pointsVector;
    
    /** The max point of players **/
    private float maxPoints;
    
    /** The max X **/
    private int maxX;
    
    /** The min X **/
    private int minX;
    
    /** The max Y **/
    private int maxY;
    
    /** The min Y **/
    private int minY;
    
    /** The image to draw curve **/
    private Image offscreen;
    
    /**
     * Creates a new instance of CurvesCanvas
     * @param playerList The players list
     * @param l The players list list
     */
    public CurvesCanvas(ArrayList playerList, ArrayList l) {
        list=l;
        this.setSize(800, 600);
        colorsVector = new Vector();
        pointsVector = new Vector();
        colorDefinition();
        this.setBackground(new java.awt.Color(255, 255, 255));
        maxPoints=getMaxPlayersPoints();
        maxX=l.size()-1;
        maxY=(int)maxPoints;
        minX=0;
        minY=0;
    }
    
    /** This method return the max points reached by players in the game
     * @return return the max points reached by players in the game
     **/
    private float getMaxPlayersPoints() {
        float max = 0;
        Iterator it = list.iterator();
        while (it.hasNext()) {
            ArrayList pl = (ArrayList)it.next();
            Iterator it2 = pl.iterator();
            while (it2.hasNext()) {
                Player p = (Player)it2.next();
                if (p.getHitPoints()>max) {
                    max=p.getHitPoints();
                }
            }
        }
        return max;
    }
    
    /** This method insert the axes in the graph
     * @param maxPts The max points of players
     * @param g The graphics
     */    
    public void drawAxes(Graphics g, int maxPts) {
        //System.out.println("Max pts = "+maxPts);
        int turnsNumber = list.size();
        //System.out.println("Turns number = "+turnsNumber);
        // 750 px x 550 px
        // y
        g.drawLine(50, 550, 50, 0);
        g.drawLine(50, 550, 49, 550);
        g.drawString("0", 20, 550);
        g.drawString(""+maxPts,5,10);
        g.drawLine(50,0,49,0);
        // x
        g.drawLine(50, 550, 800, 550);
        g.drawString("0", 50, 570);
        g.drawLine(50, 550, 50, 551);
        g.drawString(""+turnsNumber, 750, 570);
        g.drawLine(800, 550, 800, 551);
        // draw curves
        
        if (list.size()<=0) {
            return;
        }
        double pxOneTurn = 750/turnsNumber;
        double pxOnePoint = 550/maxPts;
        g.drawLine((int)(50+(turnsNumber*pxOneTurn)), 550, (int)(50+(turnsNumber*pxOneTurn)), 551);
        g.drawLine(50, (int)(maxPts*pxOnePoint), 49, (int)(maxPts*pxOnePoint));
        if ((maxPts/2)==0) {
            // paire
            g.drawLine(50, (int)((maxPts/2)*pxOnePoint), 49, (int)((maxPts/2)*pxOnePoint));
            g.drawString(""+maxPts, 20, (int)((maxPts/2)*pxOnePoint));
        } else {
            g.drawLine(50, (int)((maxPts/2+0.5)*pxOnePoint), 49, (int)((maxPts/2+0.5)*pxOnePoint));
            g.drawString(""+(int)(maxPts/2+0.5), 20, (int)((maxPts/2+0.5)*pxOnePoint));
        }
        if ((turnsNumber/2)==0) {
            // paire
            g.drawLine((int)(50+((turnsNumber/2)*pxOneTurn)), 550, (int)(50+((turnsNumber/2)*pxOneTurn)), 551);
            g.drawString(""+turnsNumber,(int)(50+((turnsNumber/2)*pxOneTurn)),570);
        } else {
            g.drawLine((int)(50+((turnsNumber/2+0.5)*pxOneTurn)), 550, (int)(50+((turnsNumber/2+0.5)*pxOneTurn)), 551);
            g.drawString(""+(int)(turnsNumber/2+0.5),(int)(50+((turnsNumber/2+0.5)*pxOneTurn)),570);
        }
        //System.out.println("pxOneTurn = "+pxOneTurn+" pxOnePoint = "+pxOnePoint);
    }
    
    /** This method assign a color to each player for his curve */    
    public void colorDefinition() {
        colorsVector.removeAllElements();
        Iterator it = ((ArrayList)(list.get(0))).iterator();
        Random random = new Random(new GregorianCalendar().get(Calendar.MILLISECOND));
        int index=0;
        while (it.hasNext()) {
            Player p = (Player)it.next();
            switch (index) {
                case 0 :
                    colorsVector.add(new ColorElement(p, Color.BLACK));
                    break;
                case 1 : 
                    colorsVector.add(new ColorElement(p, Color.BLUE));
                    break;
                case 2 : 
                    colorsVector.add(new ColorElement(p, Color.GRAY));
                    break;
                case 3 : 
                    colorsVector.add(new ColorElement(p, Color.GREEN));
                    break;
                case 4 : 
                    colorsVector.add(new ColorElement(p, Color.MAGENTA));
                    break;
                case 5 : 
                    colorsVector.add(new ColorElement(p, Color.ORANGE));
                    break;
                case 6 : 
                    colorsVector.add(new ColorElement(p, Color.PINK));
                    break;
                case 7 : 
                    colorsVector.add(new ColorElement(p, Color.YELLOW));
                    break;
                default :
                    colorsVector.add(new ColorElement(p, new Color(random.nextInt(256),random.nextInt(256),random.nextInt(256))));
                    break;
            }
            index++;
        }
    }
    
    /** returns the color associate to a player
     * @param p The player
     * @return returns the color associate to player p
     **/
    private Color getColor(Player p) {
        Iterator it = colorsVector.iterator();
        while (it.hasNext()) {
            ColorElement elt = (ColorElement)it.next();
            if (((Player)elt.getObject()).equals(p)) {
                return elt.getColor();
            }
        }
        return null;
    }
    
    /** returns the previous point associate to a player
     * @param p The player
     * @return returns the previous point associate to player p
     **/
    private Point getPreviousPoint(Player p) {
        Iterator it = pointsVector.iterator();
        while (it.hasNext()) {
            PointElement elt = (PointElement)it.next();
            if (elt.getPlayer().equals(p)) {
                return elt.getPoint();
            }
        }
        return null;
    }
    
    /** This method store the last point of the curve of a player 
     * @param p The player to update point
     * @param point The point to store
     **/    
    private void updatePreviousPoint(Player p, Point point) {
        Iterator it = pointsVector.iterator();
        while (it.hasNext()) {
            PointElement elt = (PointElement)it.next();
            if (elt.getPlayer().equals(p)) {
                elt.setPoint(point);
                return;
            }
        }
    }

    /** This method constructs the graph
     * @param g The graphics
     */    
    public void constructGraph(Graphics g) {
        // construct axes
        int maxPts = (int)getMaxPoints();
        drawAxes(g,maxPts);
        pointsVector.removeAllElements();
        //System.out.println(list.toString());
        int turnsNumber = list.size();
        Double pxOneTurn = new Double((750/turnsNumber));
        Double pxOnePoint = new Double((550/maxPts));
        Iterator it = list.iterator();
        int cpt=0;
        while (it.hasNext()) {
            ArrayList pl = (ArrayList)it.next();
            if (simulations.SimulationParameters.debug) {
                System.out.println("Analyse du tour "+cpt);
            }
            //System.out.println(pl.toString());
            Iterator it2 = pl.iterator();
            while (it2.hasNext()) {
                Player p = (Player)it2.next();
                if (simulations.SimulationParameters.debug) {
                    System.out.println("Analyse du joueur "+p.toString()+" : "+p.getHitPoints());
                    System.out.println("pxOneTurn = "+pxOneTurn.doubleValue()+" cpt = "+cpt+" * = "+(cpt*pxOneTurn.doubleValue()));
                    System.out.println("Point x= "+(int)(50+(cpt*pxOneTurn.doubleValue()))+" y= "+(int)(p.getHitPoints()*pxOnePoint.doubleValue()));
                }
                // y=50 : pts = max y=550 : pts=0
                // x=50 : tour 1 x=750 : tour=tunrsNumber
                g.setColor(getColor(p));
                if (p.getHitPoints()<=0) {
                    g.drawLine((int)(50+(cpt*pxOneTurn.doubleValue())-1), 550,(int)(50+(cpt*pxOneTurn.doubleValue())+1),550);
                    g.drawLine((int)(50+(cpt*pxOneTurn.doubleValue())), 550,(int)(50+(cpt*pxOneTurn.doubleValue())), 550);
                    // draw line between previous point and this new point
                    Point point = getPreviousPoint(p);
                    if (point!=null) {
                        g.drawLine((int)point.getX(),(int)point.getY(),(int)(50+(cpt*pxOneTurn.doubleValue())), 550);
                        updatePreviousPoint(p,new Point((int)(50+(cpt*pxOneTurn.doubleValue())), 550));
                    } else {
                        pointsVector.addElement(new PointElement(p, new Point((int)(50+(cpt*pxOneTurn.doubleValue())), 550)));
                    }
                } else {
                    g.drawLine((int)(50+(cpt*pxOneTurn.doubleValue())-1), (int)(550-p.getHitPoints()*pxOnePoint.doubleValue()),
                    (int)(50+(cpt*pxOneTurn.doubleValue())+1), (int)(550-p.getHitPoints()*pxOnePoint.doubleValue()));
                    g.drawLine((int)(50+(cpt*pxOneTurn.doubleValue())), (int)(550-p.getHitPoints()*pxOnePoint.doubleValue()-1),
                    (int)(50+(cpt*pxOneTurn.doubleValue())), (int)(550-p.getHitPoints()*pxOnePoint.doubleValue()+1));
                    Point point = getPreviousPoint(p);
                    if (point!=null) {
                        g.drawLine((int)point.getX(),(int)point.getY(),
                        (int)(50+(cpt*pxOneTurn.doubleValue())), (int)(550-p.getHitPoints()*pxOnePoint.doubleValue()));
                        updatePreviousPoint(p,new Point((int)(50+(cpt*pxOneTurn.doubleValue())), (int)(550-p.getHitPoints()*pxOnePoint.doubleValue())));
                    } else {
                        pointsVector.addElement(new PointElement(p,new Point((int)(50+(cpt*pxOneTurn.doubleValue())), (int)(550-p.getHitPoints()*pxOnePoint.doubleValue()))));
                    }
                }
            }
            cpt++;
        }
        //System.out.println("Fin constructGraph");
    }
    
    /** Overload of paintComponent method
     * @param g The graphics
     */    
    public void paint(Graphics g) {
        //constructGraph(g);
        //colorDefinition();
        offscreen = createImage( 800,600);
        // on fait notre dessin
        new CurvesFactory().drawCurves(g, list, minY, maxY, minX, maxX, colorsVector);
        //drawOffScreen( (Graphics2D)offScreen.getGraphics(), d );
        // maintenant tous ce qui suis sera dessiner par dessu le graph 
        // sans que celui ci soit redessiner
        if (offscreen!=null) {// maintenant on affiche notre image
            g.drawImage(offscreen, 0, 0, this );
        }
        //new CurvesFactory().drawCurves(g, list, minY, maxY, minX, maxX, colorsVector);
    }
    
        /** Getter for property colorsVector.
     * @return Value of property colorsVector.
     *
     */
    public java.util.Vector getColorsVector() {
        return colorsVector;
    }    
    
    /** Setter for property colorsVector.
     * @param colorsVector New value of property colorsVector.
     *
     */
    public void setColorsVector(java.util.Vector colorsVector) {
        this.colorsVector = colorsVector;
    }    

    /** Setter for property maxPoints.
     * @param maxPoints New value of property maxPoints.
     *
     */
    public void setMaxPoints(float maxPoints) {
        this.maxPoints = maxPoints;
    }
    
    /** Getter for property maxPoints.
     * @return Value of property maxPoints.
     *
     */
    public float getMaxPoints() {
        return maxPoints;
    }
    
    /** Getter for property maxX.
     * @return Value of property maxX.
     *
     */
    public int getMaxX() {
        return maxX;
    }
    
    /** Setter for property maxX.
     * @param maxX New value of property maxX.
     *
     */
    public void setMaxX(int maxX) {
        this.maxX = maxX;
    }
    
    /** Getter for property maxY.
     * @return Value of property maxY.
     *
     */
    public int getMaxY() {
        return maxY;
    }
    
    /** Setter for property maxY.
     * @param maxY New value of property maxY.
     *
     */
    public void setMaxY(int maxY) {
        this.maxY = maxY;
    }
    
    /** Getter for property minX.
     * @return Value of property minX.
     *
     */
    public int getMinX() {
        return minX;
    }
    
    /** Setter for property minX.
     * @param minX New value of property minX.
     *
     */
    public void setMinX(int minX) {
        this.minX = minX;
    }
    
    /** Getter for property minY.
     * @return Value of property minY.
     *
     */
    public int getMinY() {
        return minY;
    }
    
    /** Setter for property minY.
     * @param minY New value of property minY.
     *
     */
    public void setMinY(int minY) {
        this.minY = minY;
    }
    
    /**
     * Getter for property selectedPlayers.
     * @return Value of property selectedPlayers.
     */
    public java.util.ArrayList getSelectedPlayers() {
        return selectedPlayers;
    }
    
    /**
     * Setter for property selectedPlayers.
     * @param selectedPlayers New value of property selectedPlayers.
     */
    public void setSelectedPlayers(java.util.ArrayList selectedPlayers) {
        this.selectedPlayers = selectedPlayers;
    }
    
}
