/*
 * CurvesData.java
 *
 * Created on April 25, 2004, 12:28 AM
 */

package simulations.gui.curves;

import java.util.ArrayList;
import java.util.Vector;
import java.util.Iterator;
import java.util.Random;
import java.util.GregorianCalendar;
import java.util.Calendar;

import java.awt.Color;

import simulations.core.Player;
import simulations.core.strategy.PlayerIntElement;

/**
 * This class stores the players data to draw curves
 * @author tom
 */
public class CurvesData {
    
    private ArrayList data;
    
    /**
     * Creates a new instance of CurvesData
     * @param classement The ordoned player list
     * @param playerListList The list of the player list
     */
    public CurvesData(ArrayList classement, ArrayList playerListList) {
        Vector colorsVector = new Vector();
        data = new  ArrayList();
        colorsVector = colorDefinition(classement);
        Iterator it = classement.iterator();
        while (it.hasNext()) {
            PlayerIntElement elt = (PlayerIntElement)it.next();
            data.add(new CurvesPlayerData(elt.getPlayer(), playerListList,getColor(elt.getPlayer(),colorsVector)));
        }
    }
    
    /** returns the associated color of player p in the colors vector
     * @param p The player
     * @param colorsVector The colors vector
     * @return the associated color of player p in the colors vector
     **/
    private Color getColor(Player p, Vector colorsVector) {
        Iterator it = colorsVector.iterator();
        while (it.hasNext()) {
            ColorElement elt = (ColorElement)it.next();
            if (((Player)elt.getObject()).equals(p)) {
                return elt.getColor();
            }
        }
        return null;
    }
    
    /**
     * This method assign a color to each player for his curve
     * @param classement The ordoned list of players
     * @return returns a vector of ColorsVector
     */    
    public Vector colorDefinition(ArrayList classement) {
        Vector colorsVector = new Vector();
        Iterator it = classement.iterator();
        Random random = new Random(new GregorianCalendar().get(Calendar.MILLISECOND));
        int index=0;
        while (it.hasNext()) {
            PlayerIntElement elt = (PlayerIntElement)it.next();
            Player p = elt.getPlayer();
            switch (index) {
                case 0 :
                    colorsVector.add(new ColorElement(p, Color.BLACK));
                    break;
                case 1 : 
                    colorsVector.add(new ColorElement(p, Color.BLUE));
                    break;
                case 2 : 
                    colorsVector.add(new ColorElement(p, Color.GRAY));
                    break;
                case 3 : 
                    colorsVector.add(new ColorElement(p, Color.GREEN));
                    break;
                case 4 : 
                    colorsVector.add(new ColorElement(p, Color.MAGENTA));
                    break;
                case 5 : 
                    colorsVector.add(new ColorElement(p, Color.ORANGE));
                    break;
                case 6 : 
                    colorsVector.add(new ColorElement(p, Color.PINK));
                    break;
                case 7 : 
                    colorsVector.add(new ColorElement(p, Color.YELLOW));
                    break;
                default :
                    colorsVector.add(new ColorElement(p, new Color(random.nextInt(256),random.nextInt(256),random.nextInt(256))));
                    break;
            }
            index++;
        }
        return colorsVector;
    }
    
    
    /**
     * Return the data of a player's curves in an double array
     * @param p The player
     * @return Return the data of a player's curves in an double array
     */    
    public Double[] getData(Player p) {
        Iterator it = data.iterator();
        while (it.hasNext()) {
            CurvesPlayerData d = (CurvesPlayerData)it.next();
            if (d.getPlayer().equals(p)) {
                return d.getData();
            }
        }
        return null;
    }
    
    /**
     * Returns the curves player data of a player
     * @param p The player
     * @return Returns the curves player data of a player
     */    
    public CurvesPlayerData getCurvesPlayerData(Player p) {
        Iterator it = data.iterator();
        while (it.hasNext()) {
            CurvesPlayerData d = (CurvesPlayerData)it.next();
            if (d.getPlayer().equals(p)) {
                return d;
            }
        }
        return null;
    }
    
    /**
     * Getter for property data.
     * @return Value of property data.
     */
    public java.util.ArrayList getData() {
        return data;
    }
    
    /**
     * Setter for property data.
     * @param data New value of property data.
     */
    public void setData(java.util.ArrayList data) {
        this.data = data;
    }
    
}
