/*
 * CurvesFactory.java
 *
 * Created on April 10, 2004, 11:42 PM
 */

package simulations.gui.curves;

import java.util.ArrayList;
import java.util.Vector;
import java.util.Iterator;
import java.awt.Point;

import java.awt.Graphics;
import java.awt.Color;

import simulations.core.Player;

/**
 * This class is used to create the curves of the game by modifying the Graphics of the composant
 * @author  tom
 */
public class CurvesFactory {
    
    /** This vector contains the associated color of each player **/
    private Vector colorsVector;
    
    /** This vector contains the last point (x,y) of each player **/
    private Vector pointsVector;
    
    /** Creates a new instance of CurvesFactory */
    public CurvesFactory() {
        pointsVector = new Vector();
    }
    
    /** This method insert the axes in the graph
     * @param minX The min value in x
     * @param maxX The max value in x
     * @param minY The min value in y
     * @param maxY The max value in x
     * @param g The graphics
     */    
    public void drawAxes(Graphics g, int minX, int maxX, int minY, int maxY) {
        //System.out.println("Max pts = "+maxPts);
        //System.out.println("Turns number = "+turnsNumber);
        // 750 px x 550 px
        // y
        g.drawLine(50, 550, 50, 0);
        g.drawLine(50, 550, 49, 550);
        g.drawString(""+minY, 20, 550);
        g.drawString(""+maxY,5,10);
        g.drawLine(50,0,49,0);
        // x
        g.drawLine(50, 550, 800, 550);
        g.drawString(""+minX, 50, 570);
        g.drawLine(50, 550, 50, 551);
        g.drawString(""+maxX, 750, 570);
        g.drawLine(800, 550, 800, 551);
        // draw curves
        
        Double pxOnePoint = new Double(550/(maxY-minY));
        Double pxOneTurn = new Double(750/(maxX-minX));
        if (simulations.SimulationParameters.debug) {
            System.out.println("minX = "+minX+" maxX = "+maxX+" minY = "+minY+" maxY = "+maxY);
            System.out.println("(maxY-minY)/550 = "+((maxY-minY)/550));
            System.out.println("(maxX-minX)/750 = "+((maxX-minX)/750));
            System.out.println(""+pxOnePoint.doubleValue());
            System.out.println(""+pxOneTurn.doubleValue());
        }
        g.drawLine(800, 550, 800, 551);
        g.drawLine(50, 550, 49, 550);
        if (((maxY-minY)/2)==0) {
            // paire
            g.drawLine(50, (int)(((maxY-minY)/2)*pxOnePoint.doubleValue()), 49, (int)(((maxY-minY)/2)*pxOnePoint.doubleValue()));
            g.drawString(""+(maxY+minY)/2, 20, (int)(((maxY-minY)/2)*pxOnePoint.doubleValue()));
        } else {
            g.drawLine(50, (int)(((maxY-minY)/2+0.5)*pxOnePoint.doubleValue()), 49, (int)(((maxY-minY)/2+0.5)*pxOnePoint.doubleValue()));
            g.drawString(""+(int)((maxY+minY)/2+0.5), 20, (int)(((maxY-minY)/2+0.5)*pxOnePoint.doubleValue()));
        }
        if (((maxX-minX)/2)==0) {
            // paire
            g.drawLine((int)(50+(((maxX-minX)/2)*pxOneTurn.doubleValue())), 550, (int)(50+(((maxX-minX)/2)*pxOneTurn.doubleValue())), 551);
            g.drawString(""+(maxX+minX),(int)(50+(((maxX-minX)/2)*pxOneTurn.doubleValue())),570);
        } else {
            g.drawLine((int)(50+(((maxX-minX)/2+0.5)*pxOneTurn.doubleValue())), 550, (int)(50+(((maxX-minX)/2+0.5)*pxOneTurn.doubleValue())), 551);
            g.drawString(""+(int)((maxX+minX)/2+0.5),(int)(50+(((maxX-minX)/2+0.5)*pxOneTurn.doubleValue())),570);
        }
        //System.out.println("pxOneTurn = "+pxOneTurn+" pxOnePoint = "+pxOnePoint);
    }
    
    /** returns the color associate to a player
     * @param p The player
     * @return returns the color associate to player p
     **/
    private Color getColor(Player p) {
        Iterator it = colorsVector.iterator();
        while (it.hasNext()) {
            ColorElement elt = (ColorElement)it.next();
            if (((Player)elt.getObject()).equals(p)) {
                return elt.getColor();
            }
        }
        return null;
    }
    
    /** returns the previous point associate to a player
     * @param p The player
     * @return returns the previous point associate to player p
     **/
    private Point getPreviousPoint(Player p) {
        Iterator it = pointsVector.iterator();
        while (it.hasNext()) {
            PointElement elt = (PointElement)it.next();
            if (elt.getPlayer().equals(p)) {
                return elt.getPoint();
            }
        }
        return null;
    }
    
    /** This method store the last point of the curve of a player 
     * @param p The player to update point
     * @param point The point to store
     **/    
    private void updatePreviousPoint(Player p, Point point) {
        Iterator it = pointsVector.iterator();
        while (it.hasNext()) {
            PointElement elt = (PointElement)it.next();
            if (elt.getPlayer().equals(p)) {
                elt.setPoint(point);
                return;
            }
        }
    }
    
    /** This method creates the curves by affecting the Graphics
     * The four last parameters define the display parameters
     * @param colorsVector The colors vector to use to draw curves
     * @param g The graphics of the component
     * @param playerListList the list of the players in each turn
     * @param minPoint The min point to consider (minY)
     * @param maxPoint The max point to consider (maxY)
     * @param minTurn The min turn to consider (minX)
     * @param maxTurn The max turn to consider (maxX)
     */
    public void drawCurves(Graphics g, ArrayList playerListList, int minPoint, int maxPoint, int minTurn, int maxTurn, Vector colorsVector) {
        this.colorsVector = colorsVector;
        pointsVector.removeAllElements();
        drawAxes(g,minTurn,maxTurn,minPoint,maxPoint);
        Double pxOnePoint = new Double(550/(maxPoint-minPoint));
        Double pxOneTurn = new Double(750/(maxTurn-minTurn));
        Iterator it = playerListList.iterator();
        int cpt=0;
        // go to minTurn th turn
        while ((it.hasNext())&&(cpt<minTurn)) {
            it.next();
            cpt++;
        }
        while ((it.hasNext())&&(cpt<=(maxTurn+1))) {
            ArrayList pl = (ArrayList)it.next();
            if (simulations.SimulationParameters.debug) {
                System.out.println("Analyse du tour "+cpt);
            }
            //System.out.println(pl.toString());
            Iterator it2 = pl.iterator();
            while (it2.hasNext()) {
                Player p = (Player)it2.next();
                if ((p.getHitPoints()<minPoint)||(p.getHitPoints()>maxPoint)) {
                    continue;
                }
                if (simulations.SimulationParameters.debug) {
                    System.out.println("Analyse du joueur "+p.toString()+" : "+p.getHitPoints());
                    System.out.println("pxOneTurn = "+pxOneTurn.doubleValue()+" cpt = "+cpt+" * = "+((cpt-minTurn)*pxOneTurn.doubleValue()));
                    System.out.println("Point x= "+(int)(50+((cpt-minTurn)*pxOneTurn.doubleValue()))+" y= "+(int)((p.getHitPoints()-minPoint)*pxOnePoint.doubleValue()));
                }
                // y=50 : pts = max y=550 : pts=0
                // x=50 : tour 1 x=750 : tour=tunrsNumber
                g.setColor(getColor(p));
                if (p.getHitPoints()<=0) {
                    g.drawLine((int)(50+((cpt-minTurn)*pxOneTurn.doubleValue())-1), 550,(int)(50+((cpt-minTurn)*pxOneTurn.doubleValue())+1),550);
                    g.drawLine((int)(50+((cpt-minTurn)*pxOneTurn.doubleValue())), 550,(int)(50+((cpt-minTurn)*pxOneTurn.doubleValue())), 550);
                    // draw line between previous point and this new point
                    Point point = getPreviousPoint(p);
                    if (point!=null) {
                        g.drawLine((int)point.getX(),(int)point.getY(),(int)(50+((cpt-minTurn)*pxOneTurn.doubleValue())), 550);
                        updatePreviousPoint(p,new Point((int)(50+((cpt-minTurn)*pxOneTurn.doubleValue())), 550));
                    } else {
                        pointsVector.addElement(new PointElement(p, new Point((int)(50+((cpt-minTurn)*pxOneTurn.doubleValue())), 550)));
                    }
                } else {
                    g.drawLine((int)(50+((cpt-minTurn)*pxOneTurn.doubleValue())-1), (int)(550-(p.getHitPoints()-minPoint)*pxOnePoint.doubleValue()),
                    (int)(50+((cpt-minTurn)*pxOneTurn.doubleValue())+1), (int)(550-(p.getHitPoints()-minPoint)*pxOnePoint.doubleValue()));
                    g.drawLine((int)(50+((cpt-minTurn)*pxOneTurn.doubleValue())), (int)(550-(p.getHitPoints()-minPoint)*pxOnePoint.doubleValue()-1),
                    (int)(50+((cpt-minTurn)*pxOneTurn.doubleValue())), (int)(550-(p.getHitPoints()-minPoint)*pxOnePoint.doubleValue()+1));
                    Point point = getPreviousPoint(p);
                    if (point!=null) {
                        g.drawLine((int)point.getX(),(int)point.getY(),
                        (int)(50+((cpt-minTurn)*pxOneTurn.doubleValue())), (int)(550-(p.getHitPoints()-minPoint)*pxOnePoint.doubleValue()));
                        updatePreviousPoint(p,new Point((int)(50+((cpt-minTurn)*pxOneTurn.doubleValue())), (int)(550-(p.getHitPoints()-minPoint)*pxOnePoint.doubleValue())));
                    } else {
                        pointsVector.addElement(new PointElement(p,new Point((int)(50+((cpt-minTurn)*pxOneTurn.doubleValue())), (int)(550-(p.getHitPoints()-minPoint)*pxOnePoint.doubleValue()))));
                    }
                }
            }
            cpt++;
        }
    }
    
}
